@@ -17,7 +17,6 @@
 
 #pragma once
 
-#include <memory>
 #include <vector>
 
 #include "gutil/ref_counted.h"
@@ -27,7 +26,7 @@
 namespace doris {
 
 struct StorePath;
-class Thread;
+inline bool k_doris_exit = false;
 
 class Daemon {
 public:
@@ -46,21 +45,18 @@ public:
 
 private:
     void tcmalloc_gc_thread();
-    void buffer_pool_gc_thread();
     void memory_maintenance_thread();
     void memory_gc_thread();
     void load_channel_tracker_refresh_thread();
-    void memory_tracker_profile_refresh_thread();
     void calculate_metrics_thread();
+    void block_spill_gc_thread();
 
     CountDownLatch _stop_background_threads_latch;
     scoped_refptr<Thread> _tcmalloc_gc_thread;
-    // only buffer pool gc, will be removed after.
-    scoped_refptr<Thread> _buffer_pool_gc_thread;
     scoped_refptr<Thread> _memory_maintenance_thread;
     scoped_refptr<Thread> _memory_gc_thread;
     scoped_refptr<Thread> _load_channel_tracker_refresh_thread;
-    scoped_refptr<Thread> _memory_tracker_profile_refresh_thread;
     scoped_refptr<Thread> _calculate_metrics_thread;
+    scoped_refptr<Thread> _block_spill_gc_thread;
 };
 } // namespace doris
