@@ -187,21 +187,42 @@ Status get_int_value(const rapidjson::Value& col, PrimitiveType type, void* slot
 
 template <typename T, typename RT>
 Status get_date_value_int(const rapidjson::Value& col, PrimitiveType type, bool is_date_str,
-                          RT* slot) {
+                          RT* slot, const cctz::time_zone& time_zone) {
     constexpr bool is_datetime_v1 = std::is_same_v<T, vectorized::VecDateTimeValue>;
     T dt_val;
     if (is_date_str) {
         const std::string str_date = col.GetString();
         int str_length = col.GetStringLength();
         bool success = false;
-        // YYYY-MM-DDTHH:MM:SSZ or YYYY-MM-DDTHH:MM:SS+08:00 or 2022-08-08T12:10:10.000Z
         if (str_length > 19) {
             std::chrono::system_clock::time_point tp;
-            const bool ok =
-                    cctz::parse("%Y-%m-%dT%H:%M:%E*S%Ez", str_date, cctz::utc_time_zone(), &tp);
+            // time_zone suffix pattern
+            // Z/+08:00/-04:30
+            RE2 time_zone_pattern(R"([+-]\d{2}:\d{2}|Z)");
+            bool ok = false;
+            std::string fmt;
+            re2::StringPiece value;
+            if (time_zone_pattern.Match(str_date, 0, str_date.size(), RE2::UNANCHORED, &value, 1)) {
+                // with time_zone info
+                // YYYY-MM-DDTHH:MM:SSZ or YYYY-MM-DDTHH:MM:SS+08:00
+                // or 2022-08-08T12:10:10.000Z or YYYY-MM-DDTHH:MM:SS-08:00
+                fmt = "%Y-%m-%dT%H:%M:%E*S%Ez";
+                cctz::time_zone ctz;
+                // find time_zone by time_zone suffix string
+                TimezoneUtils::find_cctz_time_zone(value.as_string(), ctz);
+                ok = cctz::parse(fmt, str_date, ctz, &tp);
+            } else {
+                // without time_zone info
+                // 2022-08-08T12:10:10.000
+                fmt = "%Y-%m-%dT%H:%M:%E*S";
+                // If the time without time_zone info, ES will assume it is UTC time.
+                // So we parse it in Doris with UTC time zone.
+                ok = cctz::parse(fmt, str_date, cctz::utc_time_zone(), &tp);
+            }
             if (ok) {
-                success = dt_val.from_unixtime(std::chrono::system_clock::to_time_t(tp),
-                                               cctz::local_time_zone());
+                // The local time zone can change by session variable `time_zone`
+                // We should use the user specified time zone, not the actual system local time zone.
+                success = dt_val.from_unixtime(std::chrono::system_clock::to_time_t(tp), time_zone);
             }
         } else if (str_length == 19) {
             // YYYY-MM-DDTHH:MM:SS
@@ -211,7 +232,7 @@ Status get_date_value_int(const rapidjson::Value& col, PrimitiveType type, bool
                         cctz::parse("%Y-%m-%dT%H:%M:%S", str_date, cctz::utc_time_zone(), &tp);
                 if (ok) {
                     success = dt_val.from_unixtime(std::chrono::system_clock::to_time_t(tp),
-                                                   cctz::local_time_zone());
+                                                   time_zone);
                 }
             } else {
                 // YYYY-MM-DD HH:MM:SS
@@ -222,7 +243,7 @@ Status get_date_value_int(const rapidjson::Value& col, PrimitiveType type, bool
             // string long like "1677895728000"
             int64_t time_long = std::atol(str_date.c_str());
             if (time_long > 0) {
-                success = dt_val.from_unixtime(time_long / 1000, cctz::local_time_zone());
+                success = dt_val.from_unixtime(time_long / 1000, time_zone);
             }
         } else {
             // YYYY-MM-DD or others
@@ -234,7 +255,7 @@ Status get_date_value_int(const rapidjson::Value& col, PrimitiveType type, bool
         }
 
     } else {
-        if (!dt_val.from_unixtime(col.GetInt64() / 1000, cctz::local_time_zone())) {
+        if (!dt_val.from_unixtime(col.GetInt64() / 1000, time_zone)) {
             RETURN_ERROR_IF_CAST_FORMAT_ERROR(col, type);
         }
     }
@@ -251,14 +272,14 @@ Status get_date_value_int(const rapidjson::Value& col, PrimitiveType type, bool
 }
 
 template <typename T, typename RT>
-Status get_date_int(const rapidjson::Value& col, PrimitiveType type, bool pure_doc_value,
-                    RT* slot) {
+Status get_date_int(const rapidjson::Value& col, PrimitiveType type, bool pure_doc_value, RT* slot,
+                    const cctz::time_zone& time_zone) {
     // this would happend just only when `enable_docvalue_scan = false`, and field has timestamp format date from _source
     if (col.IsNumber()) {
         // ES process date/datetime field would use millisecond timestamp for index or docvalue
         // processing date type field, if a number is encountered, Doris On ES will force it to be processed according to ms
         // Doris On ES needs to be consistent with ES, so just divided by 1000 because the unit for from_unixtime is seconds
-        return get_date_value_int<T, RT>(col, type, false, slot);
+        return get_date_value_int<T, RT>(col, type, false, slot, time_zone);
     } else if (col.IsArray() && pure_doc_value && !col.Empty()) {
         // this would happened just only when `enable_docvalue_scan = true`
         // ES add default format for all field after ES 6.4, if we not provided format for `date` field ES would impose
@@ -266,22 +287,22 @@ Status get_date_int(const rapidjson::Value& col, PrimitiveType type, bool pure_d
         // At present, we just process this string format date. After some PR were merged into Doris, we would impose `epoch_mills` for
         // date field's docvalue
         if (col[0].IsString()) {
-            return get_date_value_int<T, RT>(col[0], type, true, slot);
+            return get_date_value_int<T, RT>(col[0], type, true, slot, time_zone);
         }
         // ES would return millisecond timestamp for date field, divided by 1000 because the unit for from_unixtime is seconds
-        return get_date_value_int<T, RT>(col[0], type, false, slot);
+        return get_date_value_int<T, RT>(col[0], type, false, slot, time_zone);
     } else {
         // this would happened just only when `enable_docvalue_scan = false`, and field has string format date from _source
         RETURN_ERROR_IF_COL_IS_ARRAY(col, type);
         RETURN_ERROR_IF_COL_IS_NOT_STRING(col, type);
-        return get_date_value_int<T, RT>(col, type, true, slot);
+        return get_date_value_int<T, RT>(col, type, true, slot, time_zone);
     }
 }
 template <typename T, typename RT>
 Status fill_date_int(const rapidjson::Value& col, PrimitiveType type, bool pure_doc_value,
-                     vectorized::IColumn* col_ptr) {
+                     vectorized::IColumn* col_ptr, const cctz::time_zone& time_zone) {
     RT data;
-    RETURN_IF_ERROR((get_date_int<T, RT>(col, type, pure_doc_value, &data)));
+    RETURN_IF_ERROR((get_date_int<T, RT>(col, type, pure_doc_value, &data, time_zone)));
     col_ptr->insert_data(const_cast<const char*>(reinterpret_cast<char*>(&data)), 0);
     return Status::OK();
 }
@@ -437,7 +458,8 @@ const std::string& ScrollParser::get_scroll_id() {
 Status ScrollParser::fill_columns(const TupleDescriptor* tuple_desc,
                                   std::vector<vectorized::MutableColumnPtr>& columns,
                                   bool* line_eof,
-                                  const std::map<std::string, std::string>& docvalue_context) {
+                                  const std::map<std::string, std::string>& docvalue_context,
+                                  const cctz::time_zone& time_zone) {
     *line_eof = true;
 
     if (_size <= 0 || _line_index >= _size) {
@@ -630,16 +652,17 @@ Status ScrollParser::fill_columns(const TupleDescriptor* tuple_desc,
         case TYPE_DATE:
         case TYPE_DATETIME:
             RETURN_IF_ERROR((fill_date_int<vectorized::VecDateTimeValue, int64_t>(
-                    col, type, pure_doc_value, col_ptr)));
+                    col, type, pure_doc_value, col_ptr, time_zone)));
             break;
         case TYPE_DATEV2:
             RETURN_IF_ERROR(
                     (fill_date_int<vectorized::DateV2Value<vectorized::DateV2ValueType>, uint32_t>(
-                            col, type, pure_doc_value, col_ptr)));
+                            col, type, pure_doc_value, col_ptr, time_zone)));
             break;
         case TYPE_DATETIMEV2: {
-            RETURN_IF_ERROR((fill_date_int<vectorized::DateV2Value<vectorized::DateTimeV2ValueType>,
-                                           uint64_t>(col, type, pure_doc_value, col_ptr)));
+            RETURN_IF_ERROR(
+                    (fill_date_int<vectorized::DateV2Value<vectorized::DateTimeV2ValueType>,
+                                   uint64_t>(col, type, pure_doc_value, col_ptr, time_zone)));
             break;
         }
         case TYPE_ARRAY: {
@@ -747,7 +770,8 @@ Status ScrollParser::fill_columns(const TupleDescriptor* tuple_desc,
                     uint32_t data;
                     RETURN_IF_ERROR(
                             (get_date_int<vectorized::DateV2Value<vectorized::DateV2ValueType>,
-                                          uint32_t>(sub_col, sub_type, pure_doc_value, &data)));
+                                          uint32_t>(sub_col, sub_type, pure_doc_value, &data,
+                                                    time_zone)));
                     array.push_back(data);
                     break;
                 }
@@ -755,7 +779,8 @@ Status ScrollParser::fill_columns(const TupleDescriptor* tuple_desc,
                     uint64_t data;
                     RETURN_IF_ERROR(
                             (get_date_int<vectorized::DateV2Value<vectorized::DateTimeV2ValueType>,
-                                          uint64_t>(sub_col, sub_type, pure_doc_value, &data)));
+                                          uint64_t>(sub_col, sub_type, pure_doc_value, &data,
+                                                    time_zone)));
                     array.push_back(data);
                     break;
                 }
