@@ -55,10 +55,6 @@ public:
                                                const doris::TGetTablesParams& request,
                                                TListTableMetadataNameIdsResult* result);
 
-    static Status describe_table(const std::string& ip, const int32_t port,
-                                 const TDescribeTableParams& desc_params,
-                                 TDescribeTableResult* desc_result);
-
     static Status describe_tables(const std::string& ip, const int32_t port,
                                   const TDescribeTablesParams& desc_params,
                                   TDescribeTablesResult* desc_result);
