@@ -123,6 +123,8 @@ struct VOlapTablePartition {
     int64_t num_buckets = 0;
     std::vector<OlapTableIndexTablets> indexes;
     bool is_mutable;
+    // -1 indicates load_to_single_tablet = false
+    int64_t load_tablet_idx = -1;
 
     VOlapTablePartition(vectorized::Block* partition_block)
             : start_key {partition_block, -1}, end_key {partition_block, -1} {}
@@ -182,7 +184,7 @@ private:
 
     Status _create_partition_key(const TExprNode& t_expr, BlockRow* part_key, uint16_t pos);
 
-    std::function<uint32_t(BlockRow*, int64_t)> _compute_tablet_index;
+    std::function<uint32_t(BlockRow*, const VOlapTablePartition&)> _compute_tablet_index;
 
     // check if this partition contain this key
     bool _part_contains(VOlapTablePartition* part, BlockRow* key) const {
