@@ -17,17 +17,35 @@
 
 #pragma once
 
+#include <glog/logging.h>
+#include <stddef.h>
+#include <stdint.h>
+
+#include <algorithm>
+#include <ostream>
+#include <set>
+#include <string>
+#include <utility>
 #include <vector>
 
+#include "common/status.h"
 #include "olap/column_predicate.h"
+#include "olap/olap_common.h"
+#include "vec/columns/column.h"
+
+namespace roaring {
+class Roaring;
+} // namespace roaring
 
 namespace doris {
+class WrapperField;
 
 namespace segment_v2 {
 class BloomFilter;
-}
+class InvertedIndexIterator;
+} // namespace segment_v2
 
-// Block Column Predicate support do column predicate in RowBlockV2 and support OR and AND predicate
+// Block Column Predicate support do column predicate and support OR and AND predicate
 // Block Column Predicate will replace column predicate as a unified external vectorized interface
 // in the future
 // TODO: support do predicate on Bitmap and ZoneMap, So we can use index of column to do predicate on
@@ -37,14 +55,6 @@ public:
     BlockColumnPredicate() = default;
     virtual ~BlockColumnPredicate() = default;
 
-    // evaluate all predicate on Block
-    virtual void evaluate(RowBlockV2* block, uint16_t* selected_size) const = 0;
-    // evaluate and semantics in all child block column predicate, flags as temporary variable identification
-    // to mark whether select vector is selected in evaluate the column predicate
-    virtual void evaluate_and(RowBlockV2* block, uint16_t selected_size, bool* flags) const = 0;
-    // evaluate or semantics in all child block column predicate
-    virtual void evaluate_or(RowBlockV2* block, uint16_t selected_size, bool* flags) const = 0;
-
     virtual void get_all_column_ids(std::set<ColumnId>& column_id_set) const = 0;
 
     virtual void get_all_column_predicate(
@@ -55,36 +65,45 @@ public:
         return selected_size;
     }
     virtual void evaluate_and(vectorized::MutableColumns& block, uint16_t* sel,
-                              uint16_t selected_size, bool* flags) const {};
+                              uint16_t selected_size, bool* flags) const {}
     virtual void evaluate_or(vectorized::MutableColumns& block, uint16_t* sel,
-                             uint16_t selected_size, bool* flags) const {};
+                             uint16_t selected_size, bool* flags) const {}
 
-    virtual void evaluate_vec(vectorized::MutableColumns& block, uint16_t size,
-                              bool* flags) const {};
+    virtual void evaluate_vec(vectorized::MutableColumns& block, uint16_t size, bool* flags) const {
+    }
 
     virtual bool evaluate_and(const std::pair<WrapperField*, WrapperField*>& statistic) const {
         LOG(FATAL) << "should not reach here";
         return true;
-    };
+    }
 
     virtual bool evaluate_and(const segment_v2::BloomFilter* bf) const {
         LOG(FATAL) << "should not reach here";
         return true;
-    };
-    virtual bool can_do_bloom_filter() const { return false; }
+    }
+
+    virtual bool evaluate_and(const StringRef* dict_words, const size_t dict_num) const {
+        LOG(FATAL) << "should not reach here";
+        return true;
+    }
+
+    virtual bool can_do_bloom_filter(bool ngram) const { return false; }
+
+    //evaluate predicate on inverted
+    virtual Status evaluate(const std::string& column_name, InvertedIndexIterator* iterator,
+                            uint32_t num_rows, roaring::Roaring* bitmap) const {
+        return Status::Error<ErrorCode::INVERTED_INDEX_NOT_IMPLEMENTED>(
+                "Not Implemented evaluate with inverted index, please check the predicate");
+    }
 };
 
 class SingleColumnBlockPredicate : public BlockColumnPredicate {
 public:
-    explicit SingleColumnBlockPredicate(const ColumnPredicate* pre) : _predicate(pre) {};
-
-    void evaluate(RowBlockV2* block, uint16_t* selected_size) const override;
-    void evaluate_and(RowBlockV2* block, uint16_t selected_size, bool* flags) const override;
-    void evaluate_or(RowBlockV2* block, uint16_t selected_size, bool* flags) const override;
+    explicit SingleColumnBlockPredicate(const ColumnPredicate* pre) : _predicate(pre) {}
 
     void get_all_column_ids(std::set<ColumnId>& column_id_set) const override {
         column_id_set.insert(_predicate->column_id());
-    };
+    }
 
     void get_all_column_predicate(std::set<const ColumnPredicate*>& predicate_set) const override {
         predicate_set.insert(_predicate);
@@ -96,12 +115,15 @@ public:
                       bool* flags) const override;
     bool evaluate_and(const std::pair<WrapperField*, WrapperField*>& statistic) const override;
     bool evaluate_and(const segment_v2::BloomFilter* bf) const override;
+    bool evaluate_and(const StringRef* dict_words, const size_t dict_num) const override;
     void evaluate_or(vectorized::MutableColumns& block, uint16_t* sel, uint16_t selected_size,
                      bool* flags) const override;
 
     void evaluate_vec(vectorized::MutableColumns& block, uint16_t size, bool* flags) const override;
 
-    bool can_do_bloom_filter() const override { return _predicate->can_do_bloom_filter(); }
+    bool can_do_bloom_filter(bool ngram) const override {
+        return _predicate->can_do_bloom_filter(ngram);
+    }
 
 private:
     const ColumnPredicate* _predicate;
@@ -127,7 +149,7 @@ public:
         for (auto child_block_predicate : _block_column_predicate_vec) {
             child_block_predicate->get_all_column_ids(column_id_set);
         }
-    };
+    }
 
     void get_all_column_predicate(std::set<const ColumnPredicate*>& predicate_set) const override {
         for (auto child_block_predicate : _block_column_predicate_vec) {
@@ -141,14 +163,6 @@ protected:
 
 class OrBlockColumnPredicate : public MutilColumnBlockPredicate {
 public:
-    void evaluate(RowBlockV2* block, uint16_t* selected_size) const override;
-
-    // It's kind of confusing here, when OrBlockColumnPredicate as a child of AndBlockColumnPredicate:
-    // 1.OrBlockColumnPredicate need evaluate all child BlockColumnPredicate OR SEMANTICS inside first
-    // 2.Do AND SEMANTICS in flags use 1 result to get proper select flags
-    void evaluate_and(RowBlockV2* block, uint16_t selected_size, bool* flags) const override;
-    void evaluate_or(RowBlockV2* block, uint16_t selected_size, bool* flags) const override;
-
     uint16_t evaluate(vectorized::MutableColumns& block, uint16_t* sel,
                       uint16_t selected_size) const override;
     void evaluate_and(vectorized::MutableColumns& block, uint16_t* sel, uint16_t selected_size,
@@ -161,14 +175,6 @@ public:
 
 class AndBlockColumnPredicate : public MutilColumnBlockPredicate {
 public:
-    void evaluate(RowBlockV2* block, uint16_t* selected_size) const override;
-    void evaluate_and(RowBlockV2* block, uint16_t selected_size, bool* flags) const override;
-
-    // It's kind of confusing here, when AndBlockColumnPredicate as a child of OrBlockColumnPredicate:
-    // 1.AndBlockColumnPredicate need evaluate all child BlockColumnPredicate AND SEMANTICS inside first
-    // 2.Evaluate OR SEMANTICS in flags use 1 result to get proper select flags
-    void evaluate_or(RowBlockV2* block, uint16_t selected_size, bool* flags) const override;
-
     uint16_t evaluate(vectorized::MutableColumns& block, uint16_t* sel,
                       uint16_t selected_size) const override;
     void evaluate_and(vectorized::MutableColumns& block, uint16_t* sel, uint16_t selected_size,
@@ -182,14 +188,19 @@ public:
 
     bool evaluate_and(const segment_v2::BloomFilter* bf) const override;
 
-    bool can_do_bloom_filter() const override {
+    bool evaluate_and(const StringRef* dict_words, const size_t dict_num) const override;
+
+    bool can_do_bloom_filter(bool ngram) const override {
         for (auto& pred : _block_column_predicate_vec) {
-            if (!pred->can_do_bloom_filter()) {
+            if (!pred->can_do_bloom_filter(ngram)) {
                 return false;
             }
         }
         return true;
     }
+
+    Status evaluate(const std::string& column_name, InvertedIndexIterator* iterator,
+                    uint32_t num_rows, roaring::Roaring* bitmap) const override;
 };
 
 } //namespace doris
