@@ -22,6 +22,7 @@
 #include <ostream>
 #include <string>
 
+#include "common/config.h"
 #include "common/logging.h"
 #include "olap/cumulative_compaction_time_series_policy.h"
 #include "olap/olap_common.h"
@@ -32,10 +33,11 @@ namespace doris {
 
 SizeBasedCumulativeCompactionPolicy::SizeBasedCumulativeCompactionPolicy(
         int64_t promotion_size, double promotion_ratio, int64_t promotion_min_size,
-        int64_t compaction_min_size)
+        int64_t promotion_version_count, int64_t compaction_min_size)
         : _promotion_size(promotion_size),
           _promotion_ratio(promotion_ratio),
           _promotion_min_size(promotion_min_size),
+          _promotion_version_count(promotion_version_count),
           _compaction_min_size(compaction_min_size) {}
 
 void SizeBasedCumulativeCompactionPolicy::calculate_cumulative_point(
@@ -151,6 +153,15 @@ void SizeBasedCumulativeCompactionPolicy::update_cumulative_point(
         size_t total_size = output_rowset->rowset_meta()->total_disk_size();
         if (total_size >= tablet->cumulative_promotion_size()) {
             tablet->set_cumulative_layer_point(output_rowset->end_version() + 1);
+        } else if (tablet->enable_unique_key_merge_on_write() &&
+                   output_rowset->end_version() - output_rowset->start_version() >
+                           _promotion_version_count) {
+            // for MoW table, if there's too many versions, the delete bitmap will grow to
+            // a very big size, which may cause the tablet meta too big and the `save_meta`
+            // operation too slow.
+            // if the rowset should not promotion according to it's disk size, we should also
+            // consider it's version count here.
+            tablet->set_cumulative_layer_point(output_rowset->end_version() + 1);
         }
     }
 }
@@ -234,7 +245,7 @@ int SizeBasedCumulativeCompactionPolicy::pick_input_rowsets(
         Tablet* tablet, const std::vector<RowsetSharedPtr>& candidate_rowsets,
         const int64_t max_compaction_score, const int64_t min_compaction_score,
         std::vector<RowsetSharedPtr>* input_rowsets, Version* last_delete_version,
-        size_t* compaction_score) {
+        size_t* compaction_score, bool allow_delete) {
     size_t promotion_size = tablet->cumulative_promotion_size();
     auto max_version = tablet->max_version().first;
     int transient_size = 0;
@@ -242,7 +253,7 @@ int SizeBasedCumulativeCompactionPolicy::pick_input_rowsets(
     int64_t total_size = 0;
     for (auto& rowset : candidate_rowsets) {
         // check whether this rowset is delete version
-        if (rowset->rowset_meta()->has_delete_predicate()) {
+        if (!allow_delete && rowset->rowset_meta()->has_delete_predicate()) {
             *last_delete_version = rowset->version();
             if (!input_rowsets->empty()) {
                 // we meet a delete version, and there were other versions before.
