@@ -51,7 +51,7 @@ namespace doris {
 using namespace ErrorCode;
 
 BetaRowsetReader::BetaRowsetReader(BetaRowsetSharedPtr rowset)
-        : _context(nullptr), _rowset(std::move(rowset)), _stats(&_owned_stats) {
+        : _read_context(nullptr), _rowset(std::move(rowset)), _stats(&_owned_stats) {
     _rowset->acquire();
 }
 
@@ -76,9 +76,9 @@ bool BetaRowsetReader::update_profile(RuntimeProfile* profile) {
 
 Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context,
                                                std::vector<RowwiseIteratorUPtr>* out_iters,
-                                               const RowSetSplits& rs_splits, bool use_cache) {
+                                               bool use_cache) {
     RETURN_IF_ERROR(_rowset->load());
-    _context = read_context;
+    _read_context = read_context;
     // The segment iterator is created with its own statistics,
     // and the member variable '_stats'  is initialized by '_stats(&owned_stats)'.
     // The choice of statistics used depends on the workload of the rowset reader.
@@ -88,32 +88,32 @@ Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context
     // However, for compaction or schema change workloads,
     // the read_context passed to the function will have null statistics,
     // and in such cases we will try to use the beta rowset reader's own statistics.
-    if (_context->stats != nullptr) {
-        _stats = _context->stats;
+    if (_read_context->stats != nullptr) {
+        _stats = _read_context->stats;
     }
 
     // convert RowsetReaderContext to StorageReadOptions
     _read_options.block_row_max = read_context->batch_size;
     _read_options.stats = _stats;
-    _read_options.push_down_agg_type_opt = _context->push_down_agg_type_opt;
-    _read_options.remaining_conjunct_roots = _context->remaining_conjunct_roots;
-    _read_options.common_expr_ctxs_push_down = _context->common_expr_ctxs_push_down;
+    _read_options.push_down_agg_type_opt = _read_context->push_down_agg_type_opt;
+    _read_options.remaining_conjunct_roots = _read_context->remaining_conjunct_roots;
+    _read_options.common_expr_ctxs_push_down = _read_context->common_expr_ctxs_push_down;
     _read_options.rowset_id = _rowset->rowset_id();
     _read_options.version = _rowset->version();
     _read_options.tablet_id = _rowset->rowset_meta()->tablet_id();
-    if (read_context->lower_bound_keys != nullptr) {
-        for (int i = 0; i < read_context->lower_bound_keys->size(); ++i) {
-            _read_options.key_ranges.emplace_back(&read_context->lower_bound_keys->at(i),
-                                                  read_context->is_lower_keys_included->at(i),
-                                                  &read_context->upper_bound_keys->at(i),
-                                                  read_context->is_upper_keys_included->at(i));
+    if (_read_context->lower_bound_keys != nullptr) {
+        for (int i = 0; i < _read_context->lower_bound_keys->size(); ++i) {
+            _read_options.key_ranges.emplace_back(&_read_context->lower_bound_keys->at(i),
+                                                  _read_context->is_lower_keys_included->at(i),
+                                                  &_read_context->upper_bound_keys->at(i),
+                                                  _read_context->is_upper_keys_included->at(i));
         }
     }
 
     // delete_hanlder is always set, but it maybe not init, so that it will return empty conditions
     // or predicates when it is not inited.
-    if (read_context->delete_handler != nullptr) {
-        read_context->delete_handler->get_delete_conditions_after_version(
+    if (_read_context->delete_handler != nullptr) {
+        _read_context->delete_handler->get_delete_conditions_after_version(
                 _rowset->end_version(), _read_options.delete_condition_predicates.get(),
                 &_read_options.del_predicates_for_zone_map);
     }
@@ -121,9 +121,9 @@ Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context
     std::vector<uint32_t> read_columns;
     std::set<uint32_t> read_columns_set;
     std::set<uint32_t> delete_columns_set;
-    for (int i = 0; i < _context->return_columns->size(); ++i) {
-        read_columns.push_back(_context->return_columns->at(i));
-        read_columns_set.insert(_context->return_columns->at(i));
+    for (int i = 0; i < _read_context->return_columns->size(); ++i) {
+        read_columns.push_back(_read_context->return_columns->at(i));
+        read_columns_set.insert(_read_context->return_columns->at(i));
     }
     _read_options.delete_condition_predicates->get_all_column_ids(delete_columns_set);
     for (auto cid : delete_columns_set) {
@@ -133,22 +133,23 @@ Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context
     }
     VLOG_NOTICE << "read columns size: " << read_columns.size();
     std::string schema_key = SchemaCache::get_schema_key(
-            _read_options.tablet_id, _context->tablet_schema, read_columns,
-            _context->tablet_schema->schema_version(), SchemaCache::Type::SCHEMA);
+            _read_options.tablet_id, _read_context->tablet_schema, read_columns,
+            _read_context->tablet_schema->schema_version(), SchemaCache::Type::SCHEMA);
     // It is necessary to ensure that there is a schema version when using a cache
     // because the absence of a schema version can result in reading a stale version
     // of the schema after a schema change.
-    if (_context->tablet_schema->schema_version() < 0 ||
+    if (_read_context->tablet_schema->schema_version() < 0 ||
         (_input_schema = SchemaCache::instance()->get_schema<SchemaSPtr>(schema_key)) == nullptr) {
-        _input_schema = std::make_shared<Schema>(_context->tablet_schema->columns(), read_columns);
+        _input_schema =
+                std::make_shared<Schema>(_read_context->tablet_schema->columns(), read_columns);
         SchemaCache::instance()->insert_schema(schema_key, _input_schema);
     }
 
-    if (read_context->predicates != nullptr) {
+    if (_read_context->predicates != nullptr) {
         _read_options.column_predicates.insert(_read_options.column_predicates.end(),
-                                               read_context->predicates->begin(),
-                                               read_context->predicates->end());
-        for (auto pred : *(read_context->predicates)) {
+                                               _read_context->predicates->begin(),
+                                               _read_context->predicates->end());
+        for (auto pred : *(_read_context->predicates)) {
             if (_read_options.col_id_to_predicates.count(pred->column_id()) < 1) {
                 _read_options.col_id_to_predicates.insert(
                         {pred->column_id(), std::make_shared<AndBlockColumnPredicate>()});
@@ -159,20 +160,20 @@ Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context
         }
     }
 
-    if (read_context->predicates_except_leafnode_of_andnode != nullptr) {
+    if (_read_context->predicates_except_leafnode_of_andnode != nullptr) {
         _read_options.column_predicates_except_leafnode_of_andnode.insert(
                 _read_options.column_predicates_except_leafnode_of_andnode.end(),
-                read_context->predicates_except_leafnode_of_andnode->begin(),
-                read_context->predicates_except_leafnode_of_andnode->end());
+                _read_context->predicates_except_leafnode_of_andnode->begin(),
+                _read_context->predicates_except_leafnode_of_andnode->end());
     }
 
     // Take a delete-bitmap for each segment, the bitmap contains all deletes
     // until the max read version, which is read_context->version.second
-    if (read_context->delete_bitmap != nullptr) {
+    if (_read_context->delete_bitmap != nullptr) {
         RowsetId rowset_id = rowset()->rowset_id();
         for (uint32_t seg_id = 0; seg_id < rowset()->num_segments(); ++seg_id) {
-            auto d = read_context->delete_bitmap->get_agg(
-                    {rowset_id, seg_id, read_context->version.second});
+            auto d = _read_context->delete_bitmap->get_agg(
+                    {rowset_id, seg_id, _read_context->version.second});
             if (d->isEmpty()) {
                 continue; // Empty delete bitmap for the segment
             }
@@ -183,11 +184,11 @@ Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context
     }
 
     if (_should_push_down_value_predicates()) {
-        if (read_context->value_predicates != nullptr) {
+        if (_read_context->value_predicates != nullptr) {
             _read_options.column_predicates.insert(_read_options.column_predicates.end(),
-                                                   read_context->value_predicates->begin(),
-                                                   read_context->value_predicates->end());
-            for (auto pred : *(read_context->value_predicates)) {
+                                                   _read_context->value_predicates->begin(),
+                                                   _read_context->value_predicates->end());
+            for (auto pred : *(_read_context->value_predicates)) {
                 if (_read_options.col_id_to_predicates.count(pred->column_id()) < 1) {
                     _read_options.col_id_to_predicates.insert(
                             {pred->column_id(), std::make_shared<AndBlockColumnPredicate>()});
@@ -198,26 +199,25 @@ Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context
             }
         }
     }
-    _read_options.use_page_cache = read_context->use_page_cache;
-    _read_options.tablet_schema = read_context->tablet_schema;
-    _read_options.record_rowids = read_context->record_rowids;
-    _read_options.use_topn_opt = read_context->use_topn_opt;
-    _read_options.read_orderby_key_reverse = read_context->read_orderby_key_reverse;
-    _read_options.read_orderby_key_columns = read_context->read_orderby_key_columns;
-    _read_options.io_ctx.reader_type = read_context->reader_type;
+    _read_options.use_page_cache = _read_context->use_page_cache;
+    _read_options.tablet_schema = _read_context->tablet_schema;
+    _read_options.record_rowids = _read_context->record_rowids;
+    _read_options.use_topn_opt = _read_context->use_topn_opt;
+    _read_options.read_orderby_key_reverse = _read_context->read_orderby_key_reverse;
+    _read_options.read_orderby_key_columns = _read_context->read_orderby_key_columns;
+    _read_options.io_ctx.reader_type = _read_context->reader_type;
     _read_options.io_ctx.file_cache_stats = &_stats->file_cache_stats;
-    _read_options.runtime_state = read_context->runtime_state;
-    _read_options.output_columns = read_context->output_columns;
+    _read_options.runtime_state = _read_context->runtime_state;
+    _read_options.output_columns = _read_context->output_columns;
 
     // load segments
-    // use cache is true when do vertica compaction
-    bool should_use_cache = use_cache || read_context->reader_type == ReaderType::READER_QUERY;
+    bool should_use_cache = use_cache || _read_context->reader_type == ReaderType::READER_QUERY;
     RETURN_IF_ERROR(SegmentLoader::instance()->load_segments(_rowset, &_segment_cache_handle,
                                                              should_use_cache));
 
     // create iterator for each segment
     auto& segments = _segment_cache_handle.get_segments();
-    auto [seg_start, seg_end] = rs_splits.segment_offsets;
+    auto [seg_start, seg_end] = _segment_offsets;
     if (seg_start == seg_end) {
         seg_start = 0;
         seg_end = segments.size();
@@ -241,27 +241,36 @@ Status BetaRowsetReader::get_segment_iterators(RowsetReaderContext* read_context
 }
 
 Status BetaRowsetReader::init(RowsetReaderContext* read_context, const RowSetSplits& rs_splits) {
-    _context = read_context;
-    _context->rowset_id = _rowset->rowset_id();
+    _read_context = read_context;
+    _read_context->rowset_id = _rowset->rowset_id();
+    _segment_offsets = rs_splits.segment_offsets;
+    return Status::OK();
+}
+
+Status BetaRowsetReader::_init_iterator_once() {
+    return _init_iter_once.call([this] { return _init_iterator(); });
+}
+
+Status BetaRowsetReader::_init_iterator() {
     std::vector<RowwiseIteratorUPtr> iterators;
-    RETURN_IF_ERROR(get_segment_iterators(_context, &iterators, rs_splits));
+    RETURN_IF_ERROR(get_segment_iterators(_read_context, &iterators));
 
     // merge or union segment iterator
-    if (read_context->need_ordered_result && _rowset->rowset_meta()->is_segments_overlapping()) {
+    if (_is_merge_iterator()) {
         auto sequence_loc = -1;
-        if (read_context->sequence_id_idx != -1) {
-            for (size_t loc = 0; loc < read_context->return_columns->size(); loc++) {
-                if (read_context->return_columns->at(loc) == read_context->sequence_id_idx) {
+        if (_read_context->sequence_id_idx != -1) {
+            for (size_t loc = 0; loc < _read_context->return_columns->size(); loc++) {
+                if (_read_context->return_columns->at(loc) == _read_context->sequence_id_idx) {
                     sequence_loc = loc;
                     break;
                 }
             }
         }
         _iterator = vectorized::new_merge_iterator(
-                std::move(iterators), sequence_loc, read_context->is_unique,
-                read_context->read_orderby_key_reverse, read_context->merged_rows);
+                std::move(iterators), sequence_loc, _read_context->is_unique,
+                _read_context->read_orderby_key_reverse, _read_context->merged_rows);
     } else {
-        if (read_context->read_orderby_key_reverse) {
+        if (_read_context->read_orderby_key_reverse) {
             // reverse iterators to read backward for ORDER BY key DESC
             std::reverse(iterators.begin(), iterators.end());
         }
@@ -279,6 +288,7 @@ Status BetaRowsetReader::init(RowsetReaderContext* read_context, const RowSetSpl
 
 Status BetaRowsetReader::next_block(vectorized::Block* block) {
     SCOPED_RAW_TIMER(&_stats->block_fetch_ns);
+    RETURN_IF_ERROR(_init_iterator_once());
     if (_empty) {
         return Status::Error<END_OF_FILE>("BetaRowsetReader is empty");
     }
@@ -298,6 +308,7 @@ Status BetaRowsetReader::next_block(vectorized::Block* block) {
 
 Status BetaRowsetReader::next_block_view(vectorized::BlockView* block_view) {
     SCOPED_RAW_TIMER(&_stats->block_fetch_ns);
+    RETURN_IF_ERROR(_init_iterator_once());
     do {
         auto s = _iterator->next_block_view(block_view);
         if (!s.ok()) {
@@ -318,8 +329,8 @@ bool BetaRowsetReader::_should_push_down_value_predicates() const {
     return _rowset->keys_type() == UNIQUE_KEYS &&
            (((_rowset->start_version() == 0 || _rowset->start_version() == 2) &&
              !_rowset->_rowset_meta->is_segments_overlapping() &&
-             _context->sequence_id_idx == -1) ||
-            _context->enable_unique_key_merge_on_write);
+             _read_context->sequence_id_idx == -1) ||
+            _read_context->enable_unique_key_merge_on_write);
 }
 
 Status BetaRowsetReader::get_segment_num_rows(std::vector<uint32_t>* segment_num_rows) {
