@@ -32,6 +32,7 @@
 #include "olap/rowset/rowset_reader.h"
 #include "olap/schema.h"
 #include "olap/segment_loader.h"
+#include "util/once.h"
 #include "vec/core/block.h"
 
 namespace doris {
@@ -50,11 +51,11 @@ public:
 
     Status get_segment_iterators(RowsetReaderContext* read_context,
                                  std::vector<RowwiseIteratorUPtr>* out_iters,
-                                 const RowSetSplits& rs_splits, bool use_cache = false) override;
+                                 bool use_cache = false) override;
     void reset_read_options() override;
     Status next_block(vectorized::Block* block) override;
     Status next_block_view(vectorized::BlockView* block_view) override;
-    bool support_return_data_by_ref() override { return _iterator->support_return_data_by_ref(); }
+    bool support_return_data_by_ref() override { return _is_merge_iterator(); }
 
     bool delete_flag() override { return _rowset->delete_flag(); }
 
@@ -84,10 +85,20 @@ public:
     RowsetReaderSharedPtr clone() override;
 
 private:
+    Status _init_iterator_once();
+    Status _init_iterator();
     bool _should_push_down_value_predicates() const;
+    bool _is_merge_iterator() const {
+        return _read_context->need_ordered_result &&
+               _rowset->rowset_meta()->is_segments_overlapping();
+    }
+
+    DorisCallOnce<Status> _init_iter_once;
+
+    std::pair<int, int> _segment_offsets;
 
     SchemaSPtr _input_schema;
-    RowsetReaderContext* _context;
+    RowsetReaderContext* _read_context;
     BetaRowsetSharedPtr _rowset;
 
     OlapReaderStatistics _owned_stats;
