@@ -281,10 +281,12 @@ Status FullTextIndexReader::query(OlapReaderStatistics* stats, RuntimeState* run
                     "inverted index path: {} not exist.", index_file_path.string());
         }
 
-        InvertedIndexCacheHandle inverted_index_cache_handle;
-        InvertedIndexSearcherCache::instance()->get_index_searcher(
-                _fs, index_dir.c_str(), index_file_name, &inverted_index_cache_handle, stats);
-        auto index_searcher = inverted_index_cache_handle.get_index_searcher();
+        auto get_index_search = [this, &index_dir, &index_file_name, &stats]() {
+            InvertedIndexCacheHandle inverted_index_cache_handle;
+            static_cast<void>(InvertedIndexSearcherCache::instance()->get_index_searcher(
+                    _fs, index_dir.c_str(), index_file_name, &inverted_index_cache_handle, stats));
+            return inverted_index_cache_handle.get_index_searcher();
+        };
 
         std::unique_ptr<lucene::search::Query> query;
         std::wstring field_ws = std::wstring(column_name.begin(), column_name.end());
@@ -315,6 +317,8 @@ Status FullTextIndexReader::query(OlapReaderStatistics* stats, RuntimeState* run
             } else {
                 stats->inverted_index_query_cache_miss++;
 
+                auto index_searcher = get_index_search();
+
                 term_match_bitmap = std::make_shared<roaring::Roaring>();
 
                 Status res = Status::OK();
@@ -363,6 +367,8 @@ Status FullTextIndexReader::query(OlapReaderStatistics* stats, RuntimeState* run
                 } else {
                     stats->inverted_index_query_cache_miss++;
 
+                    auto index_searcher = get_index_search();
+
                     term_match_bitmap = std::make_shared<roaring::Roaring>();
                     // unique_ptr with custom deleter
                     std::unique_ptr<lucene::index::Term, void (*)(lucene::index::Term*)> term {
@@ -396,11 +402,6 @@ Status FullTextIndexReader::query(OlapReaderStatistics* stats, RuntimeState* run
                     query_match_bitmap |= *term_match_bitmap;
                     break;
                 }
-                case InvertedIndexQueryType::EQUAL_QUERY: {
-                    SCOPED_RAW_TIMER(&stats->inverted_index_query_bitmap_op_timer);
-                    query_match_bitmap &= *term_match_bitmap;
-                    break;
-                }
                 default: {
                     return Status::Error<ErrorCode::INVERTED_INDEX_NOT_SUPPORTED>(
                             "fulltext query do not support query type other than match.");
