@@ -191,8 +191,6 @@ public:
     RowsetMetaSharedPtr acquire_rs_meta_by_version(const Version& version) const;
     void delete_stale_rs_meta_by_version(const Version& version);
     RowsetMetaSharedPtr acquire_stale_rs_meta_by_version(const Version& version) const;
-    const std::vector<RowsetMetaSharedPtr> delete_predicates() const;
-    bool version_for_delete_predicate(const Version& version);
 
     std::string full_name() const;
 
@@ -340,6 +338,13 @@ public:
     constexpr static inline uint32_t ROWSET_SENTINEL_MARK =
             std::numeric_limits<uint32_t>::max() - 1;
 
+    // When a delete bitmap is merged into tablet's delete bitmap, the version of entries in the delete bitmap
+    // will be replaced to the correspoding correct version. So before we finally merge a delete bitmap into
+    // tablet's delete bitmap we can use arbitary version number in BitmapKey. Here we define some version numbers
+    // for specific usage during this periods to avoid conflicts
+    constexpr static inline uint64_t TEMP_VERSION_COMMON = 0;
+    constexpr static inline uint64_t TEMP_VERSION_FOR_DELETE_SIGN = 1;
+
     /**
      * 
      * @param tablet_id the tablet which this delete bitmap associates with
