@@ -29,6 +29,7 @@
 #include <set>
 #include <shared_mutex>
 #include <string>
+#include <unordered_map>
 #include <utility>
 
 #include "common/logging.h"
@@ -38,6 +39,7 @@
 #include "olap/txn_manager.h"
 #include "olap/utils.h"
 #include "util/bvar_helper.h"
+#include "util/debug_points.h"
 #include "util/threadpool.h"
 
 namespace doris {
@@ -70,22 +72,19 @@ void TabletPublishStatistics::record_in_bvar() {
 }
 
 EnginePublishVersionTask::EnginePublishVersionTask(
-        const TPublishVersionRequest& publish_version_req, std::vector<TTabletId>* error_tablet_ids,
-        std::vector<TTabletId>* succ_tablet_ids,
-        std::vector<std::tuple<int64_t, int64_t, int64_t>>* discontinuous_version_tablets)
+        const TPublishVersionRequest& publish_version_req, std::set<TTabletId>* error_tablet_ids,
+        std::map<TTabletId, TVersion>* succ_tablets,
+        std::vector<std::tuple<int64_t, int64_t, int64_t>>* discontinuous_version_tablets,
+        std::map<TTableId, int64_t>* table_id_to_num_delta_rows)
         : _publish_version_req(publish_version_req),
           _error_tablet_ids(error_tablet_ids),
-          _succ_tablet_ids(succ_tablet_ids),
-          _discontinuous_version_tablets(discontinuous_version_tablets) {}
+          _succ_tablets(succ_tablets),
+          _discontinuous_version_tablets(discontinuous_version_tablets),
+          _table_id_to_num_delta_rows(table_id_to_num_delta_rows) {}
 
 void EnginePublishVersionTask::add_error_tablet_id(int64_t tablet_id) {
     std::lock_guard<std::mutex> lck(_tablet_ids_mutex);
-    _error_tablet_ids->push_back(tablet_id);
-}
-
-void EnginePublishVersionTask::add_succ_tablet_id(int64_t tablet_id) {
-    std::lock_guard<std::mutex> lck(_tablet_ids_mutex);
-    _succ_tablet_ids->push_back(tablet_id);
+    _error_tablet_ids->insert(tablet_id);
 }
 
 Status EnginePublishVersionTask::finish() {
@@ -93,10 +92,22 @@ Status EnginePublishVersionTask::finish() {
     int64_t transaction_id = _publish_version_req.transaction_id;
     OlapStopWatch watch;
     VLOG_NOTICE << "begin to process publish version. transaction_id=" << transaction_id;
+    DBUG_EXECUTE_IF("EnginePublishVersionTask.finish.random", {
+        if (rand() % 100 < (100 * dp->param("percent", 0.5))) {
+            LOG_WARNING("EnginePublishVersionTask.finish.random random failed");
+            return Status::InternalError("debug engine publish version task random failed");
+        }
+    });
+    DBUG_EXECUTE_IF("EnginePublishVersionTask.finish.wait", {
+        if (auto wait = dp->param<int>("duration", 0); wait > 0) {
+            LOG_WARNING("EnginePublishVersionTask.finish.wait wait").tag("wait ms", wait);
+            std::this_thread::sleep_for(std::chrono::milliseconds(wait));
+        }
+    });
     std::unique_ptr<ThreadPoolToken> token =
             StorageEngine::instance()->tablet_publish_txn_thread_pool()->new_token(
                     ThreadPool::ExecutionMode::CONCURRENT);
-
+    std::unordered_map<int64_t, int64_t> tablet_id_to_num_delta_rows;
     // each partition
     for (auto& par_ver_info : _publish_version_req.partition_version_infos) {
         int64_t partition_id = par_ver_info.partition_id;
@@ -128,7 +139,7 @@ Status EnginePublishVersionTask::finish() {
             // and receive fe's publish version task
             // this be must return as an error tablet
             if (rowset == nullptr) {
-                _error_tablet_ids->push_back(tablet_info.tablet_id);
+                add_error_tablet_id(tablet_info.tablet_id);
                 res = Status::Error<PUSH_ROWSET_NOT_FOUND>(
                         "could not find related rowset for tablet {}, txn id {}",
                         tablet_info.tablet_id, transaction_id);
@@ -137,7 +148,7 @@ Status EnginePublishVersionTask::finish() {
             TabletSharedPtr tablet = StorageEngine::instance()->tablet_manager()->get_tablet(
                     tablet_info.tablet_id, tablet_info.tablet_uid);
             if (tablet == nullptr) {
-                _error_tablet_ids->push_back(tablet_info.tablet_id);
+                add_error_tablet_id(tablet_info.tablet_id);
                 res = Status::Error<PUSH_TABLE_NOT_EXIST>(
                         "can't get tablet when publish version. tablet_id={}, schema_hash={}",
                         tablet_info.tablet_id, tablet_info.schema_hash);
@@ -155,26 +166,24 @@ Status EnginePublishVersionTask::finish() {
                     StorageEngine::instance()->txn_manager()->update_tablet_version_txn(
                             tablet_info.tablet_id, version.second, transaction_id);
                 }
-                Version max_version;
+                int64_t max_version;
                 TabletState tablet_state;
                 {
                     std::shared_lock rdlock(tablet->get_header_lock());
-                    max_version = tablet->max_version_unlocked();
+                    max_version = tablet->max_version_unlocked().second;
                     tablet_state = tablet->tablet_state();
                 }
-                if (tablet_state == TabletState::TABLET_RUNNING &&
-                    version.first != max_version.second + 1) {
-                    // If a tablet migrates out and back, the previously failed
-                    // publish task may retry on the new tablet, so check
-                    // whether the version exists. if not exist, then set
-                    // publish failed
-                    if (!tablet->check_version_exist(version)) {
+                if (version.first != max_version + 1) {
+                    if (tablet->check_version_exist(version)) {
+                        continue;
+                    }
+                    auto handle_version_not_continuous = [&]() {
                         add_error_tablet_id(tablet_info.tablet_id);
                         _discontinuous_version_tablets->emplace_back(
                                 partition_id, tablet_info.tablet_id, version.first);
                         res = Status::Error<PUBLISH_VERSION_NOT_CONTINUOUS>(
                                 "check_version_exist failed");
-                        int64_t missed_version = max_version.second + 1;
+                        int64_t missed_version = max_version + 1;
                         int64_t missed_txn_id =
                                 StorageEngine::instance()->txn_manager()->get_txn_by_tablet_version(
                                         tablet->tablet_id(), missed_version);
@@ -189,10 +198,27 @@ Status EnginePublishVersionTask::finish() {
                         } else {
                             LOG_EVERY_SECOND(INFO) << msg;
                         }
+                    };
+                    // The versions during the schema change period need to be also continuous
+                    if (tablet_state == TabletState::TABLET_NOTREADY) {
+                        Version max_continuous_version = {-1, 0};
+                        tablet->max_continuous_version_from_beginning(&max_continuous_version);
+                        if (max_version > 1 && version.first > max_version &&
+                            max_continuous_version.second != max_version) {
+                            handle_version_not_continuous();
+                            continue;
+                        }
+                    } else {
+                        handle_version_not_continuous();
+                        continue;
                     }
-                    continue;
                 }
             }
+
+            auto rowset_meta_ptr = rowset->rowset_meta();
+            tablet_id_to_num_delta_rows.insert(
+                    {rowset_meta_ptr->tablet_id(), rowset_meta_ptr->num_rows()});
+
             auto tablet_publish_txn_ptr = std::make_shared<TabletPublishTxnTask>(
                     this, tablet, rowset, partition_id, transaction_id, version, tablet_info);
             auto submit_st = token->submit_func([=]() { tablet_publish_txn_ptr->handle(); });
@@ -201,6 +227,7 @@ Status EnginePublishVersionTask::finish() {
     }
     token->wait();
 
+    _succ_tablets->clear();
     // check if the related tablet remained all have the version
     for (auto& par_ver_info : _publish_version_req.partition_version_infos) {
         int64_t partition_id = par_ver_info.partition_id;
@@ -208,25 +235,38 @@ Status EnginePublishVersionTask::finish() {
         std::set<TabletInfo> partition_related_tablet_infos;
         StorageEngine::instance()->tablet_manager()->get_partition_related_tablets(
                 partition_id, &partition_related_tablet_infos);
-
         Version version(par_ver_info.version, par_ver_info.version);
         for (auto& tablet_info : partition_related_tablet_infos) {
-            // has to use strict mode to check if check all tablets
-            if (!_publish_version_req.strict_mode) {
-                break;
-            }
             TabletSharedPtr tablet =
                     StorageEngine::instance()->tablet_manager()->get_tablet(tablet_info.tablet_id);
+            auto tablet_id = tablet_info.tablet_id;
             if (tablet == nullptr) {
-                add_error_tablet_id(tablet_info.tablet_id);
+                add_error_tablet_id(tablet_id);
+                _succ_tablets->erase(tablet_id);
+                LOG(WARNING) << "publish version failed on transaction, not found tablet. "
+                             << "transaction_id=" << transaction_id << ", tablet_id=" << tablet_id
+                             << ", version=" << par_ver_info.version;
             } else {
                 // check if the version exist, if not exist, then set publish failed
-                if (!tablet->check_version_exist(version)) {
-                    add_error_tablet_id(tablet_info.tablet_id);
+                if (_error_tablet_ids->find(tablet_id) == _error_tablet_ids->end()) {
+                    if (tablet->check_version_exist(version)) {
+                        // it's better to report the max continous succ version,
+                        // but it maybe time cost now.
+                        // current just report 0
+                        (*_succ_tablets)[tablet_id] = 0;
+                    } else {
+                        add_error_tablet_id(tablet_id);
+                        LOG(WARNING) << "publish version failed on transaction, tablet version not "
+                                        "exists. "
+                                     << "transaction_id=" << transaction_id
+                                     << ", tablet_id=" << tablet_id
+                                     << ", version=" << par_ver_info.version;
+                    }
                 }
             }
         }
     }
+    _calculate_tbl_num_delta_rows(tablet_id_to_num_delta_rows);
 
     if (!res.is<PUBLISH_VERSION_NOT_CONTINUOUS>()) {
         LOG(INFO) << "finish to publish version on transaction."
@@ -238,6 +278,19 @@ Status EnginePublishVersionTask::finish() {
     return res;
 }
 
+void EnginePublishVersionTask::_calculate_tbl_num_delta_rows(
+        const std::unordered_map<int64_t, int64_t>& tablet_id_to_num_delta_rows) {
+    for (const auto& kv : tablet_id_to_num_delta_rows) {
+        auto tablet = StorageEngine::instance()->tablet_manager()->get_tablet(kv.first);
+        if (!tablet) {
+            LOG(WARNING) << "cant find tablet by tablet_id=" << kv.first;
+            continue;
+        }
+        auto table_id = tablet->get_table_id();
+        (*_table_id_to_num_delta_rows)[table_id] += kv.second;
+    }
+}
+
 TabletPublishTxnTask::TabletPublishTxnTask(EnginePublishVersionTask* engine_task,
                                            TabletSharedPtr tablet, RowsetSharedPtr rowset,
                                            int64_t partition_id, int64_t transaction_id,
@@ -280,9 +333,7 @@ void TabletPublishTxnTask::handle() {
         _engine_publish_version_task->add_error_tablet_id(_tablet_info.tablet_id);
         return;
     }
-    _engine_publish_version_task->add_succ_tablet_id(_tablet_info.tablet_id);
     int64_t cost_us = MonotonicMicros() - _stats.submit_time_us;
-    // print stats if publish cost > 500ms
     g_tablet_publish_latency << cost_us;
     _stats.record_in_bvar();
     LOG(INFO) << "publish version successfully on tablet"
