@@ -106,7 +106,8 @@ Status EngineStorageMigrationTask::_check_running_txns() {
             _tablet->tablet_id(), _tablet->schema_hash(), _tablet->tablet_uid(), &partition_id,
             &transaction_ids);
     if (transaction_ids.size() > 0) {
-        return Status::InternalError("tablet {} has unfinished txns", _tablet->tablet_id());
+        return Status::Error<ErrorCode::INTERNAL_ERROR, false>("tablet {} has unfinished txns",
+                                                               _tablet->tablet_id());
     }
     return Status::OK();
 }
@@ -151,8 +152,8 @@ Status EngineStorageMigrationTask::_gen_and_write_header_to_hdr_file(
 
     // it will change rowset id and its create time
     // rowset create time is useful when load tablet from meta to check which tablet is the tablet to load
-    return SnapshotManager::instance()->convert_rowset_ids(full_path, tablet_id,
-                                                           _tablet->replica_id(), schema_hash);
+    return SnapshotManager::instance()->convert_rowset_ids(
+            full_path, tablet_id, _tablet->replica_id(), _tablet->partition_id(), schema_hash);
 }
 
 Status EngineStorageMigrationTask::_reload_tablet(const std::string& full_path) {
