@@ -163,6 +163,14 @@ public:
 
     bool is_source() const;
 
+    virtual Status collect_query_statistics(QueryStatistics* statistics) { return Status::OK(); };
+
+    virtual Status collect_query_statistics(QueryStatistics* statistics, int sender_id) {
+        return Status::OK();
+    };
+
+    virtual void set_query_statistics(std::shared_ptr<QueryStatistics>) {};
+
     virtual Status init(const TDataSink& tsink) { return Status::OK(); }
 
     // Prepare for running. (e.g. resource allocation, etc.)
@@ -276,12 +284,7 @@ public:
     Status sink(RuntimeState* state, vectorized::Block* in_block,
                 SourceState source_state) override {
         if (in_block->rows() > 0) {
-            auto st = _sink->send(state, in_block, source_state == SourceState::FINISHED);
-            // TODO: improvement: if sink returned END_OF_FILE, pipeline task can be finished
-            if (st.template is<ErrorCode::END_OF_FILE>()) {
-                return Status::OK();
-            }
-            return st;
+            return _sink->send(state, in_block, source_state == SourceState::FINISHED);
         }
         return Status::OK();
     }
@@ -304,6 +307,9 @@ public:
     Status finalize(RuntimeState* state) override { return Status::OK(); }
 
     [[nodiscard]] RuntimeProfile* get_runtime_profile() const override { return _sink->profile(); }
+    void set_query_statistics(std::shared_ptr<QueryStatistics> statistics) override {
+        _sink->set_query_statistics(statistics);
+    }
 
 protected:
     NodeType* _sink;
@@ -374,6 +380,16 @@ public:
         return _node->runtime_profile();
     }
 
+    Status collect_query_statistics(QueryStatistics* statistics) override {
+        RETURN_IF_ERROR(_node->collect_query_statistics(statistics));
+        return Status::OK();
+    }
+
+    Status collect_query_statistics(QueryStatistics* statistics, int sender_id) override {
+        RETURN_IF_ERROR(_node->collect_query_statistics(statistics, sender_id));
+        return Status::OK();
+    }
+
 protected:
     NodeType* _node;
     bool _use_projection;
