@@ -377,11 +377,12 @@ void FragmentMgr::coordinator_callback(const ReportStatusRequest& req) {
     FrontendServiceConnection coord(_exec_env->frontend_client_cache(), req.coord_addr,
                                     &coord_status);
     if (!coord_status.ok()) {
-        std::stringstream ss;
         UniqueId uid(req.query_id.hi, req.query_id.lo);
-        ss << "couldn't get a client for " << req.coord_addr << ", reason: " << coord_status;
-        LOG(WARNING) << "query_id: " << uid << ", " << ss.str();
-        req.update_fn(Status::InternalError(ss.str()));
+        std::stringstream ss;
+        req.coord_addr.printTo(ss);
+        req.update_fn(
+                Status::InternalError("query_id: {}, couldn't get a client for {}, reason is {}",
+                                      uid.to_string(), ss.str(), coord_status.to_string()));
         return;
     }
 
@@ -501,12 +502,12 @@ void FragmentMgr::coordinator_callback(const ReportStatusRequest& req) {
             coord->reportExecStatus(res, params);
         }
 
-        rpc_status = Status(Status::create(res.status));
+        rpc_status = Status::create<false>(res.status);
     } catch (TException& e) {
-        std::stringstream msg;
-        msg << "ReportExecStatus() to " << req.coord_addr << " failed:\n" << e.what();
-        LOG(WARNING) << msg.str();
-        rpc_status = Status::InternalError(msg.str());
+        std::stringstream ss;
+        req.coord_addr.printTo(ss);
+        rpc_status =
+                Status::InternalError("ReportExecStatus() to {} failed: {}", ss.str(), e.what());
     }
 
     if (!rpc_status.ok()) {
@@ -526,8 +527,8 @@ void FragmentMgr::_exec_actual(std::shared_ptr<FragmentExecState> exec_state,
 #endif
 
     LOG_INFO(func_name)
-            .tag("query_id", exec_state->query_id())
-            .tag("instance_id", exec_state->fragment_instance_id())
+            .tag("query_id", print_id(exec_state->query_id()))
+            .tag("instance_id", print_id(exec_state->fragment_instance_id()))
             .tag("pthread_id", (uintptr_t)pthread_self());
 
     Status st = exec_state->execute();
@@ -735,7 +736,7 @@ Status FragmentMgr::_get_query_ctx(const Params& params, TUniqueId query_id, boo
                 auto status = taskgroup::TaskGroupInfo::parse_group_info(params.workload_groups[0],
                                                                          &task_group_info);
                 if (status.ok()) {
-                    auto tg = taskgroup::TaskGroupManager::instance()->get_or_create_task_group(
+                    auto tg = _exec_env->task_group_manager()->get_or_create_task_group(
                             task_group_info);
                     tg->add_mem_tracker_limiter(query_ctx->query_mem_tracker);
                     query_ctx->set_task_group(tg);
@@ -1263,7 +1264,7 @@ Status FragmentMgr::apply_filterv2(const PPublishFilterRequestV2* request,
         std::shared_ptr<pipeline::PipelineFragmentContext> pip_context;
 
         RuntimeFilterMgr* runtime_filter_mgr = nullptr;
-        ObjectPool* pool;
+        ObjectPool* pool = nullptr;
         if (is_pipeline) {
             std::unique_lock<std::mutex> lock(_lock);
             auto iter = _pipeline_map.find(tfragment_instance_id);
