@@ -172,7 +172,7 @@ void LoadPathMgr::process_path(time_t now, const std::string& path, int64_t rese
         return;
     }
     LOG(INFO) << "Going to remove path. path=" << path;
-    Status status = io::global_local_filesystem()->delete_directory(path);
+    Status status = io::global_local_filesystem()->delete_directory_or_file(path);
     if (status.ok()) {
         LOG(INFO) << "Remove path success. path=" << path;
     } else {
