@@ -17,11 +17,26 @@
 
 #pragma once
 
+#include <stdint.h>
+
+#include <functional>
+#include <memory>
+#include <mutex>
+#include <vector>
+
+#include "common/status.h"
+#include "io/fs/kafka_consumer_pipe.h"
 #include "runtime/routine_load/data_consumer.h"
 #include "util/blocking_queue.hpp"
-#include "util/priority_thread_pool.hpp"
+#include "util/uid_util.h"
+#include "util/work_thread_pool.hpp"
+
+namespace RdKafka {
+class Message;
+} // namespace RdKafka
 
 namespace doris {
+class StreamLoadContext;
 
 // data consumer group saves a group of data consumers.
 // These data consumers share the same stream load pipe.
@@ -46,7 +61,10 @@ public:
     }
 
     // start all consumers
-    virtual Status start_all(StreamLoadContext* ctx) { return Status::OK(); }
+    virtual Status start_all(std::shared_ptr<StreamLoadContext> ctx,
+                             std::shared_ptr<io::KafkaConsumerPipe> kafka_pipe) {
+        return Status::OK();
+    }
 
 protected:
     UniqueId _grp_id;
@@ -68,9 +86,10 @@ public:
 
     virtual ~KafkaDataConsumerGroup();
 
-    virtual Status start_all(StreamLoadContext* ctx) override;
+    Status start_all(std::shared_ptr<StreamLoadContext> ctx,
+                     std::shared_ptr<io::KafkaConsumerPipe> kafka_pipe) override;
     // assign topic partitions to all consumers equally
-    Status assign_topic_partitions(StreamLoadContext* ctx);
+    Status assign_topic_partitions(std::shared_ptr<StreamLoadContext> ctx);
 
 private:
     // start a single consumer
