@@ -146,6 +146,10 @@ public:
         return _process_status;
     }
 
+    bool enable_faster_float_convert() const {
+        return _query_options.__isset.faster_float_convert && _query_options.faster_float_convert;
+    }
+
     // Appends error to the _error_log if there is space
     bool log_error(const std::string& error);
 
@@ -341,10 +345,7 @@ public:
         return _query_options.__isset.skip_delete_bitmap && _query_options.skip_delete_bitmap;
     }
 
-    bool enable_page_cache() const {
-        return !config::disable_storage_page_cache &&
-               (_query_options.__isset.enable_page_cache && _query_options.enable_page_cache);
-    }
+    bool enable_page_cache() const;
 
     int partitioned_hash_join_rows_threshold() const {
         if (!_query_options.__isset.partitioned_hash_join_rows_threshold) {
