@@ -10,7 +10,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+#pragma once
+
 #include <sstream>
+#include <string>
 
 #include "rapidjson/document.h"
 
