@@ -48,9 +48,9 @@ namespace doris {
  *
  */
 using int128_t = __int128;
-class DateTimeValue;
 class DecimalV2Value;
 
+template <bool is_binary_format = false>
 class MysqlRowBuffer {
 public:
     MysqlRowBuffer();
@@ -58,6 +58,10 @@ public:
 
     void reset() { _pos = _buf; }
 
+    // Prepare for binary row buffer
+    // init bitmap
+    void start_binary_row(uint32_t num_cols);
+
     // TODO(zhaochun): add signed/unsigned support
     int push_tinyint(int8_t data);
     int push_smallint(int16_t data);
@@ -68,11 +72,16 @@ public:
     int push_float(float data);
     int push_double(double data);
     int push_time(double data);
-    int push_datetime(const DateTimeValue& data);
+    int push_timev2(double data, int scale);
+    template <typename DateType>
+    int push_datetime(const DateType& data);
     int push_decimal(const DecimalV2Value& data, int round_scale);
     int push_string(const char* str, int64_t length);
     int push_null();
 
+    template <typename DateType>
+    int push_vec_datetime(DateType& data);
+
     // this function reserved size, change the pos step size, return old pos
     // Becareful when use the returned pointer.
     char* reserved(int64_t size);
@@ -115,9 +124,17 @@ public:
      */
     void close_dynamic_mode();
 
+    void set_faster_float_convert(bool faster) { _faster_float_convert = faster; }
+
 private:
     int reserve(int64_t size);
 
+    // append data into buffer
+    int append(const char* data, int64_t len);
+    // the same as mysql net_store_data
+    // the first few bytes is length, followed by data
+    int append_var_string(const char* data, int64_t len);
+
     char* _pos;
     char* _buf;
     int64_t _buf_size;
@@ -125,6 +142,10 @@ private:
 
     int _dynamic_mode;
     uint64_t _len_pos;
+    uint32_t _field_pos = 0;
+    uint32_t _field_count = 0;
+
+    bool _faster_float_convert = false;
 };
 
 } // namespace doris
