@@ -422,8 +422,8 @@ void ColumnArray::reserve(size_t n) {
 
 //please check you real need size in data column, because it's maybe need greater size when data is string column
 void ColumnArray::resize(size_t n) {
-    get_offsets().resize(n);
-    get_data().resize(n);
+    auto last_off = get_offsets().back();
+    get_offsets().resize_fill(n, last_off);
 }
 
 size_t ColumnArray::byte_size() const {
@@ -808,38 +808,6 @@ void ColumnArray::insert_indices_from(const IColumn& src, const int* indices_beg
     }
 }
 
-Status ColumnArray::filter_by_selector(const uint16_t* sel, size_t sel_size, IColumn* col_ptr) {
-    auto to = reinterpret_cast<vectorized::ColumnArray*>(col_ptr);
-    auto& to_offsets = to->get_offsets();
-
-    size_t element_size = 0;
-    size_t max_offset = 0;
-    for (size_t i = 0; i < sel_size; ++i) {
-        element_size += size_at(sel[i]);
-        max_offset = std::max(max_offset, offset_at(sel[i]));
-    }
-    if (max_offset > std::numeric_limits<uint16_t>::max()) {
-        return Status::Corruption("array elements too large than uint16_t::max");
-    }
-
-    to_offsets.reserve(to_offsets.size() + sel_size);
-    auto nested_sel = std::make_unique<uint16_t[]>(element_size);
-    size_t nested_sel_size = 0;
-    for (size_t i = 0; i < sel_size; ++i) {
-        auto row_off = offset_at(sel[i]);
-        auto row_size = size_at(sel[i]);
-        to_offsets.push_back(to_offsets.back() + row_size);
-        for (auto j = 0; j < row_size; ++j) {
-            nested_sel[nested_sel_size++] = row_off + j;
-        }
-    }
-
-    if (nested_sel_size > 0) {
-        return data->filter_by_selector(nested_sel.get(), nested_sel_size, &to->get_data());
-    }
-    return Status::OK();
-}
-
 ColumnPtr ColumnArray::replicate(const IColumn::Offsets& replicate_offsets) const {
     if (replicate_offsets.empty()) return clone_empty();
 
@@ -865,42 +833,35 @@ ColumnPtr ColumnArray::replicate(const IColumn::Offsets& replicate_offsets) cons
     return replicate_generic(replicate_offsets);
 }
 
-void ColumnArray::replicate(const uint32_t* indexs, size_t target_size, IColumn& column) const {
+void ColumnArray::replicate(const uint32_t* indices, size_t target_size, IColumn& column) const {
     if (target_size == 0) {
         return;
     }
-    auto total_size = get_offsets().size();
-    // |---------------------|-------------------------|-------------------------|
-    // [0, begin)             [begin, begin + count_sz)  [begin + count_sz, size())
-    //  do not need to copy    copy counts[n] times       do not need to copy
-    IColumn::Offsets replicate_offsets(total_size, 0);
-    // copy original data at offset n counts[n] times
-    auto begin = 0, end = 0;
-    while (begin < target_size) {
-        while (end < target_size && indexs[begin] == indexs[end]) {
-            end++;
+
+    auto& dst_col = assert_cast<ColumnArray&>(column);
+    auto& dst_data_col = dst_col.get_data();
+    auto& dst_offsets = dst_col.get_offsets();
+    dst_offsets.reserve(target_size);
+
+    PODArray<uint32> data_indices_to_replicate;
+
+    for (size_t i = 0; i < target_size; ++i) {
+        const auto index = indices[i];
+        const auto start = offset_at(index);
+        const auto length = size_at(index);
+        dst_offsets.push_back(dst_offsets.back() + length);
+        if (UNLIKELY(length == 0)) {
+            continue;
+        }
+
+        data_indices_to_replicate.reserve(data_indices_to_replicate.size() + length);
+        for (size_t j = start; j != start + length; ++j) {
+            data_indices_to_replicate.push_back(j);
         }
-        long index = indexs[begin];
-        replicate_offsets[index] = end - begin;
-        begin = end;
     }
 
-    // ignored
-    for (size_t i = 1; i < total_size; ++i) {
-        replicate_offsets[i] += replicate_offsets[i - 1];
-    }
-
-    auto rep_res = replicate(replicate_offsets);
-    if (!rep_res) {
-        LOG(WARNING) << "ColumnArray replicate failed, replicate_offsets count="
-                     << replicate_offsets.size() << ", max=" << replicate_offsets.back();
-        return;
-    }
-    auto& rep_res_arr = typeid_cast<const ColumnArray&>(*rep_res);
-
-    ColumnArray& res_arr = typeid_cast<ColumnArray&>(column);
-    res_arr.data = rep_res_arr.get_data_ptr();
-    res_arr.offsets = rep_res_arr.get_offsets_ptr();
+    get_data().replicate(data_indices_to_replicate.data(), data_indices_to_replicate.size(),
+                         dst_data_col);
 }
 
 template <typename T>
