@@ -19,7 +19,10 @@
 
 #include "common/status.h"
 #include "process_hash_table_probe.h"
+#include "runtime/thread_context.h" // IWYU pragma: keep
 #include "util/simd/bits.h"
+#include "vec/columns/column_filter_helper.h"
+#include "vec/exprs/vexpr_context.h"
 #include "vhash_join_node.h"
 
 namespace doris::vectorized {
@@ -45,7 +48,8 @@ ProcessHashTableProbe<JoinOpType>::ProcessHashTableProbe(HashJoinNode* join_node
           _rows_returned_counter(join_node->_rows_returned_counter),
           _search_hashtable_timer(join_node->_search_hashtable_timer),
           _build_side_output_timer(join_node->_build_side_output_timer),
-          _probe_side_output_timer(join_node->_probe_side_output_timer) {}
+          _probe_side_output_timer(join_node->_probe_side_output_timer),
+          _probe_process_hashtable_timer(join_node->_probe_process_hashtable_timer) {}
 
 template <int JoinOpType>
 template <bool have_other_join_conjunct>
@@ -133,8 +137,7 @@ void ProcessHashTableProbe<JoinOpType>::probe_side_output_column(
             if (all_match_one) {
                 mcol[i]->insert_range_from(*column, last_probe_index, probe_size);
             } else {
-                DCHECK_GE(_items_counts.size(), last_probe_index + probe_size);
-                column->replicate(&_items_counts[0], size, *mcol[i], last_probe_index, probe_size);
+                column->replicate(&_probe_indexs[0], size, *mcol[i]);
             }
         } else {
             mcol[i]->insert_many_defaults(size);
@@ -157,10 +160,8 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
                                                      bool is_mark_join) {
     auto& probe_index = _join_node->_probe_index;
     auto& probe_raw_ptrs = _join_node->_probe_columns;
-    if (probe_index == 0 && _items_counts.size() < probe_rows) {
-        _items_counts.resize(probe_rows);
-    }
 
+    _probe_indexs.resize(_batch_size);
     if (_build_block_rows.size() < probe_rows * PROBE_SIDE_EXPLODE_RATE) {
         _build_block_rows.resize(probe_rows * PROBE_SIDE_EXPLODE_RATE);
         _build_block_offsets.resize(probe_rows * PROBE_SIDE_EXPLODE_RATE);
@@ -175,8 +176,12 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
     KeyGetter key_getter(probe_raw_ptrs, _join_node->_probe_key_sz, nullptr);
 
     if (probe_index == 0) {
-        _arena.reset(new Arena());
-        if constexpr (IsSerializedHashTableContextTraits<KeyGetter>::value) {
+        size_t old_probe_keys_memory_usage = 0;
+        if (_arena) {
+            old_probe_keys_memory_usage = _arena->size();
+        }
+        _arena.reset(new Arena()); // TODO arena reuse by clear()?
+        if constexpr (ColumnsHashing::IsPreSerializedKeysHashMethodTraits<KeyGetter>::value) {
             if (_probe_keys.size() < probe_rows) {
                 _probe_keys.resize(probe_rows);
             }
@@ -185,10 +190,12 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
                 _probe_keys[i] =
                         serialize_keys_to_pool_contiguous(i, keys_size, probe_raw_ptrs, *_arena);
             }
+            _join_node->_probe_arena_memory_usage->add(_arena->size() -
+                                                       old_probe_keys_memory_usage);
         }
     }
 
-    if constexpr (IsSerializedHashTableContextTraits<KeyGetter>::value) {
+    if constexpr (ColumnsHashing::IsPreSerializedKeysHashMethodTraits<KeyGetter>::value) {
         key_getter.set_serialized_keys(_probe_keys.data());
     }
 
@@ -206,6 +213,11 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
     size_t probe_size = 0;
     auto& probe_row_match_iter =
             std::get<ForwardIterator<Mapped>>(_join_node->_probe_row_match_iter);
+
+    std::unique_ptr<ColumnFilterHelper> mark_column;
+    if (is_mark_join) {
+        mark_column = std::make_unique<ColumnFilterHelper>(*mcol[mcol.size() - 1]);
+    }
     {
         SCOPED_TIMER(_search_hashtable_timer);
         if constexpr (!is_right_semi_anti_join) {
@@ -216,13 +228,14 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
                     if (LIKELY(current_offset < _build_block_rows.size())) {
                         _build_block_offsets[current_offset] = probe_row_match_iter->block_offset;
                         _build_block_rows[current_offset] = probe_row_match_iter->row_num;
+                        _probe_indexs[current_offset] = probe_index;
                     } else {
                         _build_block_offsets.emplace_back(probe_row_match_iter->block_offset);
                         _build_block_rows.emplace_back(probe_row_match_iter->row_num);
+                        _probe_indexs.template emplace_back(probe_index);
                     }
                     ++current_offset;
                 }
-                _items_counts[probe_index] = current_offset;
                 all_match_one &= (current_offset == 1);
                 if (!probe_row_match_iter.ok()) {
                     ++probe_index;
@@ -236,19 +249,19 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
                 if constexpr (ignore_null && need_null_map_for_probe) {
                     if ((*null_map)[probe_index]) {
                         if constexpr (probe_all) {
-                            _items_counts[probe_index++] = (uint32_t)1;
                             // only full outer / left outer need insert the data of right table
                             if (LIKELY(current_offset < _build_block_rows.size())) {
                                 _build_block_offsets[current_offset] = -1;
                                 _build_block_rows[current_offset] = -1;
+                                _probe_indexs[current_offset] = probe_index;
                             } else {
                                 _build_block_offsets.emplace_back(-1);
                                 _build_block_rows.emplace_back(-1);
+                                _probe_indexs.template emplace_back(probe_index);
                             }
                             ++current_offset;
-                        } else {
-                            _items_counts[probe_index++] = (uint32_t)0;
                         }
+                        probe_index++;
                         all_match_one = false;
                         if constexpr (probe_all) {
                             if (current_offset >= _batch_size) {
@@ -278,9 +291,14 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
                               JoinOpType == TJoinOp::NULL_AWARE_LEFT_ANTI_JOIN) {
                     if (is_mark_join) {
                         ++current_offset;
-                        assert_cast<doris::vectorized::ColumnVector<UInt8>&>(*mcol[mcol.size() - 1])
-                                .get_data()
-                                .template push_back(!find_result.is_found());
+                        bool null_result =
+                                (need_null_map_for_probe && (*null_map)[probe_index]) ||
+                                (!find_result.is_found() && _join_node->_has_null_in_build_side);
+                        if (null_result) {
+                            mark_column->insert_null();
+                        } else {
+                            mark_column->insert_value(!find_result.is_found());
+                        }
                     } else {
                         if (!find_result.is_found()) {
                             ++current_offset;
@@ -290,9 +308,14 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
                 } else if constexpr (JoinOpType == TJoinOp::LEFT_SEMI_JOIN) {
                     if (is_mark_join) {
                         ++current_offset;
-                        assert_cast<doris::vectorized::ColumnVector<UInt8>&>(*mcol[mcol.size() - 1])
-                                .get_data()
-                                .template push_back(find_result.is_found());
+                        bool null_result =
+                                (need_null_map_for_probe && (*null_map)[probe_index]) ||
+                                (!find_result.is_found() && _join_node->_has_null_in_build_side);
+                        if (null_result) {
+                            mark_column->insert_null();
+                        } else {
+                            mark_column->insert_value(find_result.is_found());
+                        }
                     } else {
                         if (find_result.is_found()) {
                             ++current_offset;
@@ -367,7 +390,16 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
                 }
 
                 uint32_t count = (uint32_t)(current_offset - last_offset);
-                _items_counts[current_probe_index] = count;
+                if (LIKELY(current_offset < _probe_indexs.size())) {
+                    for (int i = last_offset; i < current_offset; ++i) {
+                        _probe_indexs[i] = current_probe_index;
+                    }
+                } else {
+                    for (int i = last_offset; i < _probe_indexs.size(); ++i) {
+                        _probe_indexs[i] = current_probe_index;
+                    }
+                    _probe_indexs.resize(current_offset, current_probe_index);
+                }
                 all_match_one &= (count == 1);
                 if (current_offset >= _batch_size) {
                     break;
@@ -386,7 +418,7 @@ Status ProcessHashTableProbe<JoinOpType>::do_process(HashTableType& hash_table_c
     if constexpr (JoinOpType != TJoinOp::RIGHT_SEMI_JOIN &&
                   JoinOpType != TJoinOp::RIGHT_ANTI_JOIN) {
         SCOPED_TIMER(_probe_side_output_timer);
-        RETURN_IF_CATCH_BAD_ALLOC(
+        RETURN_IF_CATCH_EXCEPTION(
                 probe_side_output_column(mcol, _join_node->_left_output_slot_flags, current_offset,
                                          last_probe_index, probe_size, all_match_one, false));
     }
@@ -403,10 +435,8 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
         Block* output_block, size_t probe_rows, bool is_mark_join) {
     auto& probe_index = _join_node->_probe_index;
     auto& probe_raw_ptrs = _join_node->_probe_columns;
-    if (probe_index == 0 && _items_counts.size() < probe_rows) {
-        _items_counts.resize(probe_rows);
-    }
     if (_build_block_rows.size() < probe_rows * PROBE_SIDE_EXPLODE_RATE) {
+        _probe_indexs.resize(probe_rows * PROBE_SIDE_EXPLODE_RATE);
         _build_block_rows.resize(probe_rows * PROBE_SIDE_EXPLODE_RATE);
         _build_block_offsets.resize(probe_rows * PROBE_SIDE_EXPLODE_RATE);
     }
@@ -419,8 +449,12 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
         KeyGetter key_getter(probe_raw_ptrs, _join_node->_probe_key_sz, nullptr);
 
         if (probe_index == 0) {
+            size_t old_probe_keys_memory_usage = 0;
+            if (_arena) {
+                old_probe_keys_memory_usage = _arena->size();
+            }
             _arena.reset(new Arena());
-            if constexpr (IsSerializedHashTableContextTraits<KeyGetter>::value) {
+            if constexpr (ColumnsHashing::IsPreSerializedKeysHashMethodTraits<KeyGetter>::value) {
                 if (_probe_keys.size() < probe_rows) {
                     _probe_keys.resize(probe_rows);
                 }
@@ -430,9 +464,11 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
                                                                        *_arena);
                 }
             }
+            _join_node->_probe_arena_memory_usage->add(_arena->size() -
+                                                       old_probe_keys_memory_usage);
         }
 
-        if constexpr (IsSerializedHashTableContextTraits<KeyGetter>::value) {
+        if constexpr (ColumnsHashing::IsPreSerializedKeysHashMethodTraits<KeyGetter>::value) {
             key_getter.set_serialized_keys(_probe_keys.data());
         }
 
@@ -459,13 +495,16 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
         auto& probe_row_match_iter =
                 std::get<ForwardIterator<Mapped>>(_join_node->_probe_row_match_iter);
         if (probe_row_match_iter.ok()) {
+            SCOPED_TIMER(_search_hashtable_timer);
             auto origin_offset = current_offset;
             for (; probe_row_match_iter.ok() && current_offset < _batch_size;
                  ++probe_row_match_iter) {
                 if (LIKELY(current_offset < _build_block_rows.size())) {
+                    _probe_indexs[current_offset] = probe_index;
                     _build_block_offsets[current_offset] = probe_row_match_iter->block_offset;
                     _build_block_rows[current_offset] = probe_row_match_iter->row_num;
                 } else {
+                    _probe_indexs.template emplace_back(probe_index);
                     _build_block_offsets.emplace_back(probe_row_match_iter->block_offset);
                     _build_block_rows.emplace_back(probe_row_match_iter->row_num);
                 }
@@ -479,7 +518,6 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
 
             row_count_from_last_probe = current_offset;
             all_match_one &= (current_offset == 1);
-            _items_counts[probe_index] = current_offset;
             if (!probe_row_match_iter.ok()) {
                 ++probe_index;
                 is_the_last_sub_block = true;
@@ -488,26 +526,27 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
         }
         int multi_matched_output_row_count = 0;
         if (current_offset < _batch_size) {
+            SCOPED_TIMER(_search_hashtable_timer);
             while (probe_index < probe_rows) {
                 // ignore null rows
                 if constexpr (ignore_null && need_null_map_for_probe) {
                     if ((*null_map)[probe_index]) {
                         if constexpr (probe_all) {
-                            _items_counts[probe_index++] = (uint32_t)1;
                             same_to_prev.emplace_back(false);
                             visited_map.emplace_back(nullptr);
                             // only full outer / left outer need insert the data of right table
                             if (LIKELY(current_offset < _build_block_rows.size())) {
+                                _probe_indexs[current_offset] = probe_index;
                                 _build_block_offsets[current_offset] = -1;
                                 _build_block_rows[current_offset] = -1;
                             } else {
+                                _probe_indexs.template emplace_back(probe_index);
                                 _build_block_offsets.emplace_back(-1);
                                 _build_block_rows.emplace_back(-1);
                             }
                             ++current_offset;
-                        } else {
-                            _items_counts[probe_index++] = (uint32_t)0;
                         }
+                        probe_index++;
                         all_match_one = false;
                         if constexpr (probe_all) {
                             if (current_offset >= _batch_size) {
@@ -637,7 +676,16 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
                     ++probe_index;
                 }
                 uint32_t count = (uint32_t)(current_offset - last_offset);
-                _items_counts[current_probe_index] = count;
+                if (LIKELY(current_offset < _probe_indexs.size())) {
+                    for (int i = last_offset; i < current_offset; ++i) {
+                        _probe_indexs[i] = current_probe_index;
+                    }
+                } else {
+                    for (int i = last_offset; i < _probe_indexs.size(); ++i) {
+                        _probe_indexs[i] = current_probe_index;
+                    }
+                    _probe_indexs.resize(current_offset, current_probe_index);
+                }
                 all_match_one &= (count == 1);
                 if (current_offset >= _batch_size) {
                     break;
@@ -653,7 +701,7 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
         }
         {
             SCOPED_TIMER(_probe_side_output_timer);
-            RETURN_IF_CATCH_BAD_ALLOC(probe_side_output_column(
+            RETURN_IF_CATCH_EXCEPTION(probe_side_output_column(
                     mcol, _join_node->_left_output_slot_flags, current_offset, last_probe_index,
                     probe_size, all_match_one, true));
         }
@@ -663,11 +711,21 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
         // dispose the other join conjunct exec
         auto row_count = output_block->rows();
         if (row_count) {
-            int result_column_id = -1;
+            SCOPED_TIMER(_join_node->_process_other_join_conjunct_timer);
             int orig_columns = output_block->columns();
-            RETURN_IF_ERROR((*_join_node->_vother_join_conjunct_ptr)
-                                    ->execute(output_block, &result_column_id));
+            IColumn::Filter other_conjunct_filter(row_count, 1);
+            bool can_be_filter_all;
+            RETURN_IF_ERROR(VExprContext::execute_conjuncts(
+                    _join_node->_other_join_conjuncts, nullptr, output_block,
+                    &other_conjunct_filter, &can_be_filter_all));
 
+            auto result_column_id = output_block->columns();
+            auto filter_column = ColumnVector<UInt8>::create();
+            if (can_be_filter_all) {
+                memset(other_conjunct_filter.data(), 0, row_count);
+            }
+            filter_column->get_data() = std::move(other_conjunct_filter);
+            output_block->insert({std::move(filter_column), std::make_shared<DataTypeUInt8>(), ""});
             auto column = output_block->get_by_position(result_column_id).column;
             if constexpr (JoinOpType == TJoinOp::LEFT_OUTER_JOIN ||
                           JoinOpType == TJoinOp::FULL_OUTER_JOIN) {
@@ -798,20 +856,20 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
                 }
 
                 if (is_mark_join) {
-                    auto& matched_map = assert_cast<doris::vectorized::ColumnVector<UInt8>&>(
-                                                *(output_block->get_by_position(num_cols - 1)
-                                                          .column->assume_mutable()))
-                                                .get_data();
+                    /// FIXME: incorrect result of semi mark join with other conjuncts(null value missed).
+                    auto mark_column =
+                            output_block->get_by_position(num_cols - 1).column->assume_mutable();
+                    ColumnFilterHelper helper(*mark_column);
 
                     // For mark join, we only filter rows which have duplicate join keys.
                     // And then, we set matched_map to the join result to do the mark join's filtering.
                     for (size_t i = 1; i < row_count; ++i) {
                         if (!same_to_prev[i]) {
-                            matched_map.push_back(filter_map[i - 1]);
+                            helper.insert_value(filter_map[i - 1]);
                             filter_map[i - 1] = true;
                         }
                     }
-                    matched_map.push_back(filter_map[filter_map.size() - 1]);
+                    helper.insert_value(filter_map[filter_map.size() - 1]);
                     filter_map[filter_map.size() - 1] = true;
                 }
 
@@ -871,17 +929,20 @@ Status ProcessHashTableProbe<JoinOpType>::do_process_with_other_join_conjuncts(
                 }
 
                 if (is_mark_join) {
-                    auto& matched_map = assert_cast<doris::vectorized::ColumnVector<UInt8>&>(
-                                                *(output_block->get_by_position(num_cols - 1)
-                                                          .column->assume_mutable()))
-                                                .get_data();
-                    for (int i = 1; i < row_count; ++i) {
+                    /// FIXME: incorrect result of semi mark join with other conjuncts(null value missed).
+                    auto mark_column =
+                            output_block->get_by_position(num_cols - 1).column->assume_mutable();
+                    ColumnFilterHelper helper(*mark_column);
+
+                    // For mark join, we only filter rows which have duplicate join keys.
+                    // And then, we set matched_map to the join result to do the mark join's filtering.
+                    for (size_t i = 1; i < row_count; ++i) {
                         if (!same_to_prev[i]) {
-                            matched_map.push_back(!filter_map[i - 1]);
+                            helper.insert_value(filter_map[i - 1]);
                             filter_map[i - 1] = true;
                         }
                     }
-                    matched_map.push_back(!filter_map[row_count - 1]);
+                    helper.insert_value(filter_map[row_count - 1]);
                     filter_map[row_count - 1] = true;
                 } else {
                     int end_row_idx;
@@ -1022,6 +1083,7 @@ Status ProcessHashTableProbe<JoinOpType>::process_data_in_hashtable(HashTableTyp
                                                                     Block* output_block,
                                                                     bool* eos) {
     using Mapped = typename HashTableType::Mapped;
+    SCOPED_TIMER(_probe_process_hashtable_timer);
     if constexpr (std::is_same_v<Mapped, RowRefListWithFlag> ||
                   std::is_same_v<Mapped, RowRefListWithFlags>) {
         hash_table_ctx.init_once();
@@ -1047,8 +1109,24 @@ Status ProcessHashTableProbe<JoinOpType>::process_data_in_hashtable(HashTableTyp
         };
 
         if (visited_iter.ok()) {
-            for (; visited_iter.ok() && block_size < _batch_size; ++visited_iter) {
-                insert_from_hash_table(visited_iter->block_offset, visited_iter->row_num);
+            if constexpr (std::is_same_v<Mapped, RowRefListWithFlag>) {
+                for (; visited_iter.ok() && block_size < _batch_size; ++visited_iter) {
+                    insert_from_hash_table(visited_iter->block_offset, visited_iter->row_num);
+                }
+            } else {
+                for (; visited_iter.ok() && block_size < _batch_size; ++visited_iter) {
+                    if constexpr (JoinOpType == TJoinOp::RIGHT_SEMI_JOIN) {
+                        if (visited_iter->visited) {
+                            insert_from_hash_table(visited_iter->block_offset,
+                                                   visited_iter->row_num);
+                        }
+                    } else {
+                        if (!visited_iter->visited) {
+                            insert_from_hash_table(visited_iter->block_offset,
+                                                   visited_iter->row_num);
+                        }
+                    }
+                }
             }
             if (!visited_iter.ok()) {
                 ++iter;
@@ -1059,16 +1137,16 @@ Status ProcessHashTableProbe<JoinOpType>::process_data_in_hashtable(HashTableTyp
             auto& mapped = iter->get_second();
             if constexpr (std::is_same_v<Mapped, RowRefListWithFlag>) {
                 if (mapped.visited) {
-                    visited_iter = mapped.begin();
-                    for (; visited_iter.ok() && block_size < _batch_size; ++visited_iter) {
-                        if constexpr (JoinOpType == TJoinOp::RIGHT_SEMI_JOIN) {
+                    if constexpr (JoinOpType == TJoinOp::RIGHT_SEMI_JOIN) {
+                        visited_iter = mapped.begin();
+                        for (; visited_iter.ok() && block_size < _batch_size; ++visited_iter) {
                             insert_from_hash_table(visited_iter->block_offset,
                                                    visited_iter->row_num);
                         }
-                    }
-                    if (visited_iter.ok()) {
-                        // block_size >= _batch_size, quit for loop
-                        break;
+                        if (visited_iter.ok()) {
+                            // block_size >= _batch_size, quit for loop
+                            break;
+                        }
                     }
                 } else {
                     if constexpr (JoinOpType != TJoinOp::RIGHT_SEMI_JOIN) {
