@@ -17,13 +17,27 @@
 
 #include "vec/exec/vassert_num_rows_node.h"
 
-#include "gen_cpp/PlanNodes_types.h"
-#include "gutil/strings/substitute.h"
-#include "runtime/row_batch.h"
+#include <gen_cpp/PlanNodes_types.h>
+#include <glog/logging.h>
+#include <opentelemetry/nostd/shared_ptr.h>
+
+#include <functional>
+#include <map>
+#include <memory>
+#include <ostream>
+#include <utility>
+#include <vector>
+
 #include "runtime/runtime_state.h"
 #include "util/runtime_profile.h"
+#include "util/telemetry/telemetry.h"
 #include "vec/core/block.h"
 
+namespace doris {
+class DescriptorTbl;
+class ObjectPool;
+} // namespace doris
+
 namespace doris::vectorized {
 
 VAssertNumRowsNode::VAssertNumRowsNode(ObjectPool* pool, const TPlanNode& tnode,
@@ -39,7 +53,6 @@ VAssertNumRowsNode::VAssertNumRowsNode(ObjectPool* pool, const TPlanNode& tnode,
 }
 
 Status VAssertNumRowsNode::open(RuntimeState* state) {
-    START_AND_SCOPE_SPAN(state->get_tracer(), span, "VAssertNumRowsNode::open");
     SCOPED_TIMER(_runtime_profile->total_time_counter());
     RETURN_IF_ERROR(ExecNode::open(state));
     // ISSUE-3435
@@ -47,12 +60,7 @@ Status VAssertNumRowsNode::open(RuntimeState* state) {
     return Status::OK();
 }
 
-Status VAssertNumRowsNode::get_next(RuntimeState* state, Block* block, bool* eos) {
-    INIT_AND_SCOPE_GET_NEXT_SPAN(state->get_tracer(), _get_next_span,
-                                 "VAssertNumRowsNode::get_next");
-    SCOPED_TIMER(_runtime_profile->total_time_counter());
-    RETURN_IF_ERROR_AND_CHECK_SPAN(child(0)->get_next_after_projects(state, block, eos),
-                                   child(0)->get_next_span(), *eos);
+Status VAssertNumRowsNode::pull(doris::RuntimeState* state, vectorized::Block* block, bool* eos) {
     _num_rows_returned += block->rows();
     bool assert_res = false;
     switch (_assertion) {
@@ -98,4 +106,16 @@ Status VAssertNumRowsNode::get_next(RuntimeState* state, Block* block, bool* eos
     return Status::OK();
 }
 
+Status VAssertNumRowsNode::get_next(RuntimeState* state, Block* block, bool* eos) {
+    SCOPED_TIMER(_runtime_profile->total_time_counter());
+    RETURN_IF_ERROR(child(0)->get_next_after_projects(
+            state, block, eos,
+            std::bind((Status(ExecNode::*)(RuntimeState*, vectorized::Block*, bool*)) &
+                              ExecNode::get_next,
+                      _children[0], std::placeholders::_1, std::placeholders::_2,
+                      std::placeholders::_3)));
+
+    return pull(state, block, eos);
+}
+
 } // namespace doris::vectorized
