@@ -62,6 +62,8 @@ public:
         int64_t _load_jar_timer = 0;
         int64_t _init_connector_timer = 0;
         int64_t _get_data_timer = 0;
+        int64_t _call_jni_next_timer = 0;
+        int64_t _convert_batch_timer = 0;
         int64_t _check_type_timer = 0;
         int64_t _execte_read_timer = 0;
         int64_t _connector_close_timer = 0;
@@ -104,6 +106,8 @@ private:
                                  int rows);
     Status _cast_string_to_hll(const SlotDescriptor* slot_desc, Block* block, int column_index,
                                int rows);
+    Status _cast_string_to_bitmap(const SlotDescriptor* slot_desc, Block* block, int column_index,
+                                  int rows);
     Status _cast_string_to_json(const SlotDescriptor* slot_desc, Block* block, int column_index,
                                 int rows);
     Status _convert_batch_result_set(JNIEnv* env, jobject jobj, const SlotDescriptor* slot_desc,
@@ -146,6 +150,7 @@ private:
     jmethodID _executor_get_array_result;
     jmethodID _executor_get_json_result;
     jmethodID _executor_get_hll_result;
+    jmethodID _executor_get_bitmap_result;
     jmethodID _executor_get_types_id;
     jmethodID _executor_close_id;
     jmethodID _executor_get_list_id;
@@ -163,6 +168,10 @@ private:
     std::vector<DataTypePtr> _input_hll_string_types;
     std::vector<MutableColumnPtr> str_hll_cols; // for hll type to save data like string
 
+    std::map<int, int> _map_column_idx_to_cast_idx_bitmap;
+    std::vector<DataTypePtr> _input_bitmap_string_types;
+    std::vector<MutableColumnPtr> str_bitmap_cols; // for bitmap type to save data like string
+
     std::map<int, int> _map_column_idx_to_cast_idx_json;
     std::vector<DataTypePtr> _input_json_string_types;
     std::vector<MutableColumnPtr> str_json_cols; // for json type to save data like string
