@@ -65,6 +65,11 @@ struct NullPresence {
     bool has_null_constant = false;
 };
 
+template <typename T>
+concept HasGetVariadicArgumentTypesImpl = requires(T t) {
+    { t.get_variadic_argument_types_impl() } -> std::same_as<DataTypes>;
+};
+
 NullPresence get_null_presence(const Block& block, const ColumnNumbers& args);
 [[maybe_unused]] NullPresence get_null_presence(const ColumnsWithTypeAndName& args);
 
