@@ -379,21 +379,23 @@ public:
                     static_cast<const ColumnNullable&>(array_column.get_data());
             const auto& nested_column = nested_nullable_column.get_nested_column();
             const auto& nested_null_map = nested_nullable_column.get_null_map_column().get_data();
-            if (check_column<ColumnInt8>(nested_column)) {
-                Impl::template vector<ColumnInt8>(offset_column_data, nested_column,
-                                                  nested_null_map, res, null_map);
-            } else if (check_column<ColumnUInt8>(nested_column)) {
-                Impl::template vector<ColumnUInt8>(offset_column_data, nested_column,
-                                                   nested_null_map, res, null_map);
-            } else if (check_column<ColumnInt16>(nested_column)) {
-                Impl::template vector<ColumnInt16>(offset_column_data, nested_column,
-                                                   nested_null_map, res, null_map);
-            } else if (check_column<ColumnInt32>(nested_column)) {
-                Impl::template vector<ColumnInt32>(offset_column_data, nested_column,
-                                                   nested_null_map, res, null_map);
-            } else if (check_column<ColumnInt64>(nested_column)) {
-                Impl::template vector<ColumnInt64>(offset_column_data, nested_column,
-                                                   nested_null_map, res, null_map);
+
+            WhichDataType which_type(argument_type);
+            if (which_type.is_int8()) {
+                static_cast<void>(Impl::template vector<ColumnInt8>(
+                        offset_column_data, nested_column, nested_null_map, res, null_map));
+            } else if (which_type.is_uint8()) {
+                static_cast<void>(Impl::template vector<ColumnUInt8>(
+                        offset_column_data, nested_column, nested_null_map, res, null_map));
+            } else if (which_type.is_int16()) {
+                static_cast<void>(Impl::template vector<ColumnInt16>(
+                        offset_column_data, nested_column, nested_null_map, res, null_map));
+            } else if (which_type.is_int32()) {
+                static_cast<void>(Impl::template vector<ColumnInt32>(
+                        offset_column_data, nested_column, nested_null_map, res, null_map));
+            } else if (which_type.is_int64()) {
+                static_cast<void>(Impl::template vector<ColumnInt64>(
+                        offset_column_data, nested_column, nested_null_map, res, null_map));
             } else {
                 return Status::RuntimeError("Illegal column {} of argument of function {}",
                                             block.get_by_position(arguments[0]).column->get_name(),
@@ -582,7 +584,7 @@ struct BitmapAndNot {
             mid_data &= rvec[i];
             res[i] = lvec[i];
             res[i] -= mid_data;
-            mid_data.clear();
+            mid_data.reset();
         }
     }
     static void vector_scalar(const TData& lvec, const BitmapValue& rval, TData& res) {
@@ -593,7 +595,7 @@ struct BitmapAndNot {
             mid_data &= rval;
             res[i] = lvec[i];
             res[i] -= mid_data;
-            mid_data.clear();
+            mid_data.reset();
         }
     }
     static void scalar_vector(const BitmapValue& lval, const TData& rvec, TData& res) {
@@ -604,7 +606,7 @@ struct BitmapAndNot {
             mid_data &= rvec[i];
             res[i] = lval;
             res[i] -= mid_data;
-            mid_data.clear();
+            mid_data.reset();
         }
     }
 };
@@ -628,7 +630,7 @@ struct BitmapAndNotCount {
             mid_data = lvec[i];
             mid_data &= rvec[i];
             res[i] = lvec[i].andnot_cardinality(mid_data);
-            mid_data.clear();
+            mid_data.reset();
         }
     }
     static void scalar_vector(const BitmapValue& lval, const TData& rvec, ResTData* res) {
@@ -638,7 +640,7 @@ struct BitmapAndNotCount {
             mid_data = lval;
             mid_data &= rvec[i];
             res[i] = lval.andnot_cardinality(mid_data);
-            mid_data.clear();
+            mid_data.reset();
         }
     }
     static void vector_scalar(const TData& lvec, const BitmapValue& rval, ResTData* res) {
@@ -648,7 +650,7 @@ struct BitmapAndNotCount {
             mid_data = lvec[i];
             mid_data &= rval;
             res[i] = lvec[i].andnot_cardinality(mid_data);
-            mid_data.clear();
+            mid_data.reset();
         }
     }
 };
