@@ -21,10 +21,9 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <cstdint>
 #include <map>
 #include <memory>
-#include <mutex>
-#include <shared_mutex>
 #include <string>
 #include <type_traits>
 #include <utility>
@@ -55,22 +54,9 @@
 #include "vec/data_types/data_type_time_v2.h"
 #include "vec/functions/function.h"
 #include "vec/io/io_helper.h"
-
-namespace doris {
-namespace vectorized {
-class DataTypeDate;
-class VecDateTimeValue;
-struct DateTimeV2ValueType;
-struct DateV2ValueType;
-template <typename T>
-class DateV2Value;
-} // namespace vectorized
-} // namespace doris
-
+#include "vec/runtime/vdatetime_value.h"
 namespace doris::vectorized {
 
-using ZoneList = std::unordered_map<std::string, cctz::time_zone>;
-
 template <typename DateValueType, typename ArgType>
 struct ConvertTZImpl {
     using ColumnType = std::conditional_t<
@@ -92,8 +78,6 @@ struct ConvertTZImpl {
                         const ColumnString* from_tz_column, const ColumnString* to_tz_column,
                         ReturnColumnType* result_column, NullMap& result_null_map,
                         size_t input_rows_count) {
-        ZoneList& time_zone_cache = context->state()->exec_env()->global_zone_cache();
-        std::shared_mutex& cache_lock = context->state()->exec_env()->zone_cache_rw_lock();
         for (size_t i = 0; i < input_rows_count; i++) {
             if (result_null_map[i]) {
                 result_column->insert_default();
@@ -101,8 +85,7 @@ struct ConvertTZImpl {
             }
             auto from_tz = from_tz_column->get_data_at(i).to_string();
             auto to_tz = to_tz_column->get_data_at(i).to_string();
-            execute_inner_loop(date_column, time_zone_cache, cache_lock, from_tz, to_tz,
-                               result_column, result_null_map, i);
+            execute_inner_loop(date_column, from_tz, to_tz, result_column, result_null_map, i);
         }
     }
 
@@ -110,9 +93,6 @@ struct ConvertTZImpl {
                                  const ColumnString* from_tz_column,
                                  const ColumnString* to_tz_column, ReturnColumnType* result_column,
                                  NullMap& result_null_map, size_t input_rows_count) {
-        ZoneList& time_zone_cache = context->state()->exec_env()->global_zone_cache();
-        std::shared_mutex& cache_lock = context->state()->exec_env()->zone_cache_rw_lock();
-
         auto from_tz = from_tz_column->get_data_at(0).to_string();
         auto to_tz = to_tz_column->get_data_at(0).to_string();
         for (size_t i = 0; i < input_rows_count; i++) {
@@ -120,61 +100,58 @@ struct ConvertTZImpl {
                 result_column->insert_default();
                 continue;
             }
-            execute_inner_loop(date_column, time_zone_cache, cache_lock, from_tz, to_tz,
-                               result_column, result_null_map, i);
+            execute_inner_loop(date_column, from_tz, to_tz, result_column, result_null_map, i);
         }
     }
 
-    static void execute_inner_loop(const ColumnType* date_column, ZoneList& time_zone_cache,
-                                   std::shared_mutex& cache_lock, const std::string& from_tz,
-                                   const std::string& to_tz, ReturnColumnType* result_column,
+    static void execute_inner_loop(const ColumnType* date_column, const std::string& from_tz_name,
+                                   const std::string& to_tz_name, ReturnColumnType* result_column,
                                    NullMap& result_null_map, const size_t index_now) {
         DateValueType ts_value =
                 binary_cast<NativeType, DateValueType>(date_column->get_element(index_now));
-        int64_t timestamp;
-
-        cache_lock.lock_shared();
-        if (time_zone_cache.find(from_tz) == time_zone_cache.cend()) {
-            cache_lock.unlock_shared();
-            std::unique_lock<std::shared_mutex> lock_(cache_lock);
-            //TODO: the lock upgrade could be done in find_... function only when we push value into the hashmap
-            if (!TimezoneUtils::find_cctz_time_zone(from_tz, time_zone_cache[from_tz])) {
-                time_zone_cache.erase(from_tz);
-                result_null_map[index_now] = true;
-                result_column->insert_default();
-                return;
-            }
-        } else {
-            cache_lock.unlock_shared();
-        }
-
-        cache_lock.lock_shared();
-        if (time_zone_cache.find(to_tz) == time_zone_cache.cend()) {
-            cache_lock.unlock_shared();
-            std::unique_lock<std::shared_mutex> lock_(cache_lock);
-            if (!TimezoneUtils::find_cctz_time_zone(to_tz, time_zone_cache[to_tz])) {
-                time_zone_cache.erase(to_tz);
-                result_null_map[index_now] = true;
-                result_column->insert_default();
-                return;
-            }
-        } else {
-            cache_lock.unlock_shared();
-        }
-
-        if (!ts_value.unix_timestamp(&timestamp, time_zone_cache[from_tz])) {
-            result_null_map[index_now] = true;
-            result_column->insert_default();
-            return;
-        }
-
+        cctz::time_zone from_tz {}, to_tz {};
         ReturnDateType ts_value2;
-        if (!ts_value2.from_unixtime(timestamp, time_zone_cache[to_tz])) {
+
+        if (!TimezoneUtils::find_cctz_time_zone(from_tz_name, from_tz)) {
             result_null_map[index_now] = true;
             result_column->insert_default();
             return;
         }
 
+        if (!TimezoneUtils::find_cctz_time_zone(to_tz_name, to_tz)) {
+            result_null_map[index_now] = true;
+            result_column->insert_default();
+            return;
+        }
+
+        if constexpr (std::is_same_v<DateValueType, DateV2Value<DateTimeV2ValueType>>) {
+            std::pair<int64_t, int64_t> timestamp;
+            if (!ts_value.unix_timestamp(&timestamp, from_tz)) {
+                result_null_map[index_now] = true;
+                result_column->insert_default();
+                return;
+            }
+
+            if (!ts_value2.from_unixtime(timestamp, to_tz)) {
+                result_null_map[index_now] = true;
+                result_column->insert_default();
+                return;
+            }
+        } else {
+            int64_t timestamp;
+            if (!ts_value.unix_timestamp(&timestamp, from_tz)) {
+                result_null_map[index_now] = true;
+                result_column->insert_default();
+                return;
+            }
+
+            if (!ts_value2.from_unixtime(timestamp, to_tz)) {
+                result_null_map[index_now] = true;
+                result_column->insert_default();
+                return;
+            }
+        }
+
         result_column->insert(binary_cast<ReturnDateType, ReturnNativeType>(ts_value2));
     }
 
