@@ -17,13 +17,17 @@
 
 #include "vec/runtime/vdata_stream_mgr.h"
 
-#include "gen_cpp/internal_service.pb.h"
-#include "runtime/descriptors.h"
-#include "runtime/primitive_type.h"
-#include "runtime/raw_value.h"
-#include "runtime/runtime_state.h"
-#include "util/doris_metrics.h"
-#include "util/runtime_profile.h"
+#include <gen_cpp/Types_types.h>
+#include <gen_cpp/internal_service.pb.h>
+#include <gen_cpp/types.pb.h>
+#include <stddef.h>
+
+#include <ostream>
+#include <string>
+#include <vector>
+
+#include "common/logging.h"
+#include "util/hash_util.hpp"
 #include "vec/runtime/vdata_stream_recvr.h"
 
 namespace doris {
@@ -35,26 +39,32 @@ VDataStreamMgr::VDataStreamMgr() {
 
 VDataStreamMgr::~VDataStreamMgr() {
     // TODO: metric
+    auto receiver_iterator = _receiver_map.begin();
+    while (receiver_iterator != _receiver_map.end()) {
+        // Has to call close here, because receiver will check if the receiver is closed.
+        // It will core during graceful stop.
+        receiver_iterator->second->close();
+    }
 }
 
 inline uint32_t VDataStreamMgr::get_hash_value(const TUniqueId& fragment_instance_id,
                                                PlanNodeId node_id) {
-    uint32_t value = RawValue::get_hash_value(&fragment_instance_id.lo, TYPE_BIGINT, 0);
-    value = RawValue::get_hash_value(&fragment_instance_id.hi, TYPE_BIGINT, value);
-    value = RawValue::get_hash_value(&node_id, TYPE_INT, value);
+    uint32_t value = HashUtil::hash(&fragment_instance_id.lo, 8, 0);
+    value = HashUtil::hash(&fragment_instance_id.hi, 8, value);
+    value = HashUtil::hash(&node_id, 4, value);
     return value;
 }
 
 std::shared_ptr<VDataStreamRecvr> VDataStreamMgr::create_recvr(
         RuntimeState* state, const RowDescriptor& row_desc, const TUniqueId& fragment_instance_id,
-        PlanNodeId dest_node_id, int num_senders, int buffer_size, RuntimeProfile* profile,
-        bool is_merging, std::shared_ptr<QueryStatisticsRecvr> sub_plan_query_statistics_recvr) {
-    DCHECK(profile != NULL);
+        PlanNodeId dest_node_id, int num_senders, RuntimeProfile* profile, bool is_merging,
+        std::shared_ptr<QueryStatisticsRecvr> sub_plan_query_statistics_recvr) {
+    DCHECK(profile != nullptr);
     VLOG_FILE << "creating receiver for fragment=" << fragment_instance_id
               << ", node=" << dest_node_id;
     std::shared_ptr<VDataStreamRecvr> recvr(new VDataStreamRecvr(
-            this, state->instance_mem_tracker(), row_desc, fragment_instance_id, dest_node_id,
-            num_senders, is_merging, buffer_size, profile, sub_plan_query_statistics_recvr));
+            this, state, row_desc, fragment_instance_id, dest_node_id, num_senders, is_merging,
+            profile, sub_plan_query_statistics_recvr));
     uint32_t hash_value = get_hash_value(fragment_instance_id, dest_node_id);
     std::lock_guard<std::mutex> l(_lock);
     _fragment_stream_set.insert(std::make_pair(fragment_instance_id, dest_node_id));
@@ -67,8 +77,10 @@ std::shared_ptr<VDataStreamRecvr> VDataStreamMgr::find_recvr(const TUniqueId& fr
                                                              bool acquire_lock) {
     VLOG_ROW << "looking up fragment_instance_id=" << fragment_instance_id << ", node=" << node_id;
     size_t hash_value = get_hash_value(fragment_instance_id, node_id);
+    // Create lock guard and not own lock currently and will lock conditionally
+    std::unique_lock recvr_lock(_lock, std::defer_lock);
     if (acquire_lock) {
-        _lock.lock();
+        recvr_lock.lock();
     }
     std::pair<StreamMap::iterator, StreamMap::iterator> range =
             _receiver_map.equal_range(hash_value);
@@ -76,16 +88,10 @@ std::shared_ptr<VDataStreamRecvr> VDataStreamMgr::find_recvr(const TUniqueId& fr
         auto recvr = range.first->second;
         if (recvr->fragment_instance_id() == fragment_instance_id &&
             recvr->dest_node_id() == node_id) {
-            if (acquire_lock) {
-                _lock.unlock();
-            }
             return recvr;
         }
         ++range.first;
     }
-    if (acquire_lock) {
-        _lock.unlock();
-    }
     return std::shared_ptr<VDataStreamRecvr>();
 }
 
@@ -102,9 +108,13 @@ Status VDataStreamMgr::transmit_block(const PTransmitDataParams* request,
         // As a consequence, find_recvr() may return an innocuous NULL if a thread
         // calling deregister_recvr() beat the thread calling find_recvr()
         // in acquiring _lock.
+        //
+        // e.g. for broadcast join build side, only one instance will build the hash table,
+        // all other instances don't need build side data and will close the data stream receiver.
+        //
         // TODO: Rethink the lifecycle of DataStreamRecvr to distinguish
         // errors from receiver-initiated teardowns.
-        return Status::OK();
+        return Status::EndOfFile("data stream receiver closed");
     }
 
     // request can only be used before calling recvr's add_batch or when request
@@ -116,8 +126,9 @@ Status VDataStreamMgr::transmit_block(const PTransmitDataParams* request,
 
     bool eos = request->eos();
     if (request->has_block()) {
-        recvr->add_block(request->block(), request->sender_id(), request->be_number(),
-                         request->packet_seq(), eos ? nullptr : done);
+        RETURN_IF_ERROR(recvr->add_block(request->block(), request->sender_id(),
+                                         request->be_number(), request->packet_seq(),
+                                         eos ? nullptr : done));
     }
 
     if (eos) {
@@ -171,7 +182,7 @@ void VDataStreamMgr::cancel(const TUniqueId& fragment_instance_id) {
                 _fragment_stream_set.lower_bound(std::make_pair(fragment_instance_id, 0));
         while (i != _fragment_stream_set.end() && i->first == fragment_instance_id) {
             std::shared_ptr<VDataStreamRecvr> recvr = find_recvr(i->first, i->second, false);
-            if (recvr == NULL) {
+            if (recvr == nullptr) {
                 // keep going but at least log it
                 std::stringstream err;
                 err << "cancel(): missing in stream_map: fragment=" << i->first
