@@ -268,6 +268,7 @@ public class ColumnType {
                 type = Type.DATEV2;
                 break;
             case "binary":
+            case "bytes":
                 type = Type.BINARY;
                 break;
             case "string":
@@ -279,27 +280,27 @@ public class ColumnType {
                     precision = 6; // default
                     Matcher match = digitPattern.matcher(lowerCaseType);
                     if (match.find()) {
-                        precision = Integer.parseInt(match.group(1));
+                        precision = Integer.parseInt(match.group(1).trim());
                     }
                 } else if (lowerCaseType.startsWith("char")) {
                     Matcher match = digitPattern.matcher(lowerCaseType);
                     if (match.find()) {
                         type = Type.CHAR;
-                        length = Integer.parseInt(match.group(1));
+                        length = Integer.parseInt(match.group(1).trim());
                     }
                 } else if (lowerCaseType.startsWith("varchar")) {
                     Matcher match = digitPattern.matcher(lowerCaseType);
                     if (match.find()) {
                         type = Type.VARCHAR;
-                        length = Integer.parseInt(match.group(1));
+                        length = Integer.parseInt(match.group(1).trim());
                     }
                 } else if (lowerCaseType.startsWith("decimal")) {
                     int s = lowerCaseType.indexOf('(');
                     int e = lowerCaseType.indexOf(')');
                     if (s != -1 && e != -1) {
                         String[] ps = lowerCaseType.substring(s + 1, e).split(",");
-                        precision = Integer.parseInt(ps[0]);
-                        scale = Integer.parseInt(ps[1]);
+                        precision = Integer.parseInt(ps[0].trim());
+                        scale = Integer.parseInt(ps[1].trim());
                         if (lowerCaseType.startsWith("decimalv2")) {
                             type = Type.DECIMALV2;
                         } else if (lowerCaseType.startsWith("decimal32")) {
@@ -333,8 +334,9 @@ public class ColumnType {
                         String keyValue = lowerCaseType.substring(4, lowerCaseType.length() - 1);
                         int index = findNextNestedField(keyValue);
                         if (index != keyValue.length() && index != 0) {
-                            ColumnType keyType = parseType("key", keyValue.substring(0, index));
-                            ColumnType valueType = parseType("value", keyValue.substring(index + 1));
+                            ColumnType keyType = parseType("key", keyValue.substring(0, index).trim());
+                            ColumnType valueType =
+                                    parseType("value", keyValue.substring(index + 1).trim());
                             ColumnType mapType = new ColumnType(columnName, Type.MAP);
                             mapType.setChildTypes(Arrays.asList(keyType, valueType));
                             return mapType;
