@@ -20,6 +20,9 @@ package org.apache.doris.paimon;
 import org.apache.doris.common.jni.vec.ColumnType;
 import org.apache.doris.common.jni.vec.ColumnValue;
 
+import org.apache.paimon.data.DataGetters;
+import org.apache.paimon.data.InternalArray;
+import org.apache.paimon.data.InternalMap;
 import org.apache.paimon.data.InternalRow;
 
 import java.math.BigDecimal;
@@ -32,12 +35,18 @@ import java.util.List;
 
 public class PaimonColumnValue implements ColumnValue {
     private int idx;
-    private InternalRow record;
-    ColumnType dorisType;
+    private DataGetters record;
+    private ColumnType dorisType;
 
     public PaimonColumnValue() {
     }
 
+    public PaimonColumnValue(DataGetters record, int idx, ColumnType columnType) {
+        this.idx = idx;
+        this.record = record;
+        this.dorisType = columnType;
+    }
+
     public void setIdx(int idx, ColumnType dorisType) {
         this.idx = idx;
         this.dorisType = dorisType;
@@ -130,12 +139,29 @@ public class PaimonColumnValue implements ColumnValue {
 
     @Override
     public void unpackArray(List<ColumnValue> values) {
-
+        InternalArray recordArray = record.getArray(idx);
+        for (int i = 0; i < recordArray.size(); i++) {
+            PaimonColumnValue arrayColumnValue = new PaimonColumnValue((DataGetters) recordArray, i,
+                    dorisType.getChildTypes().get(0));
+            values.add(arrayColumnValue);
+        }
     }
 
     @Override
     public void unpackMap(List<ColumnValue> keys, List<ColumnValue> values) {
-
+        InternalMap map = record.getMap(idx);
+        InternalArray key = map.keyArray();
+        for (int i = 0; i < key.size(); i++) {
+            PaimonColumnValue keyColumnValue = new PaimonColumnValue((DataGetters) key, i,
+                    dorisType.getChildTypes().get(0));
+            keys.add(keyColumnValue);
+        }
+        InternalArray value = map.valueArray();
+        for (int i = 0; i < value.size(); i++) {
+            PaimonColumnValue valueColumnValue = new PaimonColumnValue((DataGetters) value, i,
+                    dorisType.getChildTypes().get(1));
+            values.add(valueColumnValue);
+        }
     }
 
     @Override
