@@ -34,6 +34,7 @@ import org.apache.doris.journal.bdbje.BDBTool;
 import org.apache.doris.journal.bdbje.BDBToolOptions;
 import org.apache.doris.persist.meta.MetaReader;
 import org.apache.doris.qe.QeService;
+import org.apache.doris.qe.SimpleScheduler;
 import org.apache.doris.service.ExecuteEnv;
 import org.apache.doris.service.FeServer;
 import org.apache.doris.service.FrontendOptions;
@@ -192,6 +193,8 @@ public class DorisFE {
                 Env.getCurrentEnv().setHttpReady(true);
             }
 
+            SimpleScheduler.init();
+
             if (options.enableQeService) {
                 QeService qeService = new QeService(Config.query_port, ExecuteEnv.getInstance().getScheduler());
                 qeService.start();
@@ -448,7 +451,7 @@ public class DorisFE {
             releaseFileLockAndCloseFileChannel();
             throw new RuntimeException("Try to lock process failed", e);
         }
-        throw new RuntimeException("FE process has been started，please do not start multiple FE processes at the"
+        throw new RuntimeException("FE process has been started，please do not start multiple FE processes at the "
                 + "same time");
     }
 
