@@ -23,13 +23,15 @@ import org.apache.doris.common.AnalysisException;
 import org.apache.doris.thrift.TExprNode;
 import org.apache.doris.thrift.TExprNodeType;
 
-import org.apache.commons.lang.StringUtils;
+import com.google.common.base.Preconditions;
+import org.apache.commons.lang3.StringUtils;
 
 import java.io.DataInput;
 import java.io.DataOutput;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Objects;
 
 public class ArrayLiteral extends LiteralExpr {
 
@@ -60,19 +62,26 @@ public class ArrayLiteral extends LiteralExpr {
         type = new ArrayType(itemType, containsNull);
 
         children = new ArrayList<>();
-        for (LiteralExpr expr : exprs) {
-            if (expr.getType().equals(itemType)) {
-                children.add(expr);
-            } else {
-                children.add(expr.castTo(itemType));
+        try {
+            for (LiteralExpr expr : exprs) {
+                if (expr.getType().equals(itemType)) {
+                    children.add(expr);
+                } else {
+                    children.add(expr.convertTo(itemType));
+                }
             }
+        } catch (AnalysisException e) {
+            String s = "[" + StringUtils.join(exprs, ',') + "]";
+            throw new AnalysisException("Invalid ARRAY " + s + " literal: " + e.getMessage());
         }
+        analysisDone();
     }
 
     protected ArrayLiteral(ArrayLiteral other) {
         super(other);
     }
 
+
     @Override
     public boolean isMinValue() {
         return false;
@@ -103,7 +112,7 @@ public class ArrayLiteral extends LiteralExpr {
     public String getStringValue() {
         List<String> list = new ArrayList<>(children.size());
         children.forEach(v -> list.add(v.getStringValue()));
-        return "ARRAY[" + StringUtils.join(list, ", ") + "]";
+        return "[" + StringUtils.join(list, ", ") + "]";
     }
 
     @Override
@@ -119,6 +128,23 @@ public class ArrayLiteral extends LiteralExpr {
         msg.setChildType(((ArrayType) type).getItemType().getPrimitiveType().toThrift());
     }
 
+    @Override
+    public int hashCode() {
+        return Objects.hashCode(children);
+    }
+
+    @Override
+    public boolean equals(Object o) {
+        if (!(o instanceof ArrayLiteral)) {
+            return false;
+        }
+        if (this == o) {
+            return true;
+        }
+        ArrayLiteral that = (ArrayLiteral) o;
+        return Objects.equals(children, that.children);
+    }
+
     @Override
     public void write(DataOutput out) throws IOException {
         super.write(out);
@@ -149,15 +175,32 @@ public class ArrayLiteral extends LiteralExpr {
         return new ArrayLiteral(this);
     }
 
+    @Override
+    public LiteralExpr convertTo(Type targetType) throws AnalysisException {
+        Preconditions.checkState(targetType instanceof ArrayType);
+        Type itemType = ((ArrayType) targetType).getItemType();
+        LiteralExpr[] literals = new LiteralExpr[children.size()];
+        for (int i = 0; i < children.size(); i++) {
+            literals[i] = (LiteralExpr) (Expr.convertLiteral(children.get(i), itemType));
+        }
+        return new ArrayLiteral(literals);
+    }
+
     @Override
     public Expr uncheckedCastTo(Type targetType) throws AnalysisException {
         if (!targetType.isArrayType()) {
             return super.uncheckedCastTo(targetType);
         }
+        Type itemType = ((ArrayType) targetType).getItemType();
         ArrayLiteral literal = new ArrayLiteral(this);
         for (int i = 0; i < children.size(); ++ i) {
-            Expr child = children.get(i);
-            literal.children.set(i, child.uncheckedCastTo(((ArrayType) targetType).getItemType()));
+            Expr child = Expr.convertLiteral(children.get(i), itemType);
+            // all children should be literal or else it will make be core
+            if (!child.isLiteral()) {
+                throw new AnalysisException("Unexpected array literal cast failed. from type: "
+                        + this.type + ", to type: " + targetType);
+            }
+            literal.children.set(i, child);
         }
         literal.setType(targetType);
         return literal;
