@@ -36,6 +36,8 @@ import org.apache.doris.catalog.Tablet;
 import org.apache.doris.catalog.View;
 import org.apache.doris.common.io.Text;
 import org.apache.doris.common.util.TimeUtils;
+import org.apache.doris.datasource.property.S3ClientBEProperties;
+import org.apache.doris.persist.BarrierLog;
 import org.apache.doris.task.AgentBatchTask;
 import org.apache.doris.task.AgentTask;
 import org.apache.doris.task.AgentTaskExecutor;
@@ -376,7 +378,7 @@ public class BackupJob extends AbstractJob {
                         OlapTable olapTable = (OlapTable) tbl;
                         checkOlapTable(olapTable, tableRef);
                         if (getContent() == BackupContent.ALL) {
-                            prepareSnapshotTaskForOlapTableWithoutLock((OlapTable) tbl, tableRef, batchTask);
+                            prepareSnapshotTaskForOlapTableWithoutLock(db, (OlapTable) tbl, tableRef, batchTask);
                         }
                         prepareBackupMetaForOlapTableWithoutLock(tableRef, olapTable, copiedTables);
                         break;
@@ -430,10 +432,15 @@ public class BackupJob extends AbstractJob {
         }
     }
 
-    private void prepareSnapshotTaskForOlapTableWithoutLock(OlapTable olapTable,
+    private void prepareSnapshotTaskForOlapTableWithoutLock(Database db, OlapTable olapTable,
             TableRef backupTableRef, AgentBatchTask batchTask) {
         // Add barrier editolog for barrier commit seq
-        long commitSeq = env.getEditLog().logBarrier();
+        long dbId = db.getId();
+        String dbName = db.getFullName();
+        long tableId = olapTable.getId();
+        String tableName = olapTable.getName();
+        BarrierLog barrierLog = new BarrierLog(dbId, dbName, tableId, tableName);
+        long commitSeq = env.getEditLog().logBarrier(barrierLog);
         // format as "table:{tableId}"
         String tableKey = String.format("%s%d", TABLE_COMMIT_SEQ_PREFIX, olapTable.getId());
         properties.put(tableKey, String.valueOf(commitSeq));
@@ -621,9 +628,9 @@ public class BackupJob extends AbstractJob {
                 }
                 long signature = env.getNextId();
                 UploadTask task = new UploadTask(null, beId, signature, jobId, dbId, srcToDest,
-                        brokers.get(0), repo.getRemoteFileSystem().getProperties(),
+                        brokers.get(0),
+                        S3ClientBEProperties.getBeFSProperties(repo.getRemoteFileSystem().getProperties()),
                         repo.getRemoteFileSystem().getStorageType(), repo.getLocation());
-                LOG.info("yy debug upload location: " + repo.getLocation());
                 batchTask.addTask(task);
                 unfinishedTaskIds.put(signature, beId);
             }
@@ -657,9 +664,10 @@ public class BackupJob extends AbstractJob {
 
     private void saveMetaInfo() {
         String createTimeStr = TimeUtils.longToTimeString(createTime, TimeUtils.DATETIME_FORMAT_WITH_HYPHEN);
-        // local job dir: backup/label__createtime/
+        // local job dir: backup/repo__repo_id/label__createtime/
+        // Add repo_id to isolate jobs from different repos.
         localJobDirPath = Paths.get(BackupHandler.BACKUP_ROOT_DIR.toString(),
-                                    label + "__" + createTimeStr).normalize();
+                                    "repo__" + repoId, label + "__" + createTimeStr).normalize();
 
         try {
             // 1. create local job dir of this backup job
@@ -1007,3 +1015,4 @@ public class BackupJob extends AbstractJob {
         return sb.toString();
     }
 }
+
