@@ -23,9 +23,11 @@ import org.apache.doris.common.MetaNotFoundException;
 import org.apache.doris.statistics.AnalysisInfo;
 import org.apache.doris.statistics.BaseAnalysisTask;
 import org.apache.doris.statistics.ColumnStatistic;
+import org.apache.doris.statistics.TableStatsMeta;
 import org.apache.doris.thrift.TTableDescriptor;
 
 import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -33,6 +35,7 @@ import java.io.DataOutput;
 import java.io.IOException;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
@@ -136,8 +139,25 @@ public interface TableIf {
 
     Optional<ColumnStatistic> getColumnStatistic(String colName);
 
+    boolean needReAnalyzeTable(TableStatsMeta tblStats);
+
+    Map<String, Set<String>> findReAnalyzeNeededPartitions();
+
+    // Get all the chunk sizes of this table. Now, only HMS external table implemented this interface.
+    // For HMS external table, the return result is a list of all the files' size.
+    List<Long> getChunkSizes();
+
     void write(DataOutput out) throws IOException;
 
+    /**
+     * return true if this kind of table need read lock when doing query plan.
+     *
+     * @return
+     */
+    default boolean needReadLockWhenPlan() {
+        return false;
+    }
+
     /**
      * Doris table type.
      */
@@ -236,5 +256,22 @@ public interface TableIf {
     default long getLastUpdateTime() {
         return -1L;
     }
+
+    default long getDataSize(boolean singleReplica) {
+        // TODO: Each tableIf should impl it by itself.
+        return 0;
+    }
+
+    default boolean isDistributionColumn(String columnName) {
+        return false;
+    }
+
+    default boolean isPartitionColumn(String columnName) {
+        return false;
+    }
+
+    default Set<String> getDistributionColumnNames() {
+        return Sets.newHashSet();
+    }
 }
 
