@@ -21,19 +21,22 @@ import org.apache.doris.common.AnalysisException;
 import org.apache.doris.common.AuthenticationException;
 import org.apache.doris.common.Config;
 import org.apache.doris.common.DdlException;
+import org.apache.doris.common.Pair;
 import org.apache.doris.common.profile.MultiProfileTreeBuilder;
 import org.apache.doris.common.profile.ProfileTreeBuilder;
 import org.apache.doris.common.profile.ProfileTreeNode;
+import org.apache.doris.common.profile.SummaryProfile;
+import org.apache.doris.nereids.stats.StatsErrorEstimator;
 
 import com.google.common.base.Strings;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
+import com.google.gson.Gson;
+import com.google.gson.GsonBuilder;
 import org.apache.commons.lang3.tuple.Triple;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
-import java.util.Arrays;
-import java.util.Collections;
 import java.util.Deque;
 import java.util.Iterator;
 import java.util.LinkedList;
@@ -56,61 +59,46 @@ import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;
 public class ProfileManager {
     private static final Logger LOG = LogManager.getLogger(ProfileManager.class);
     private static volatile ProfileManager INSTANCE = null;
-    // private static final int ARRAY_SIZE = 100;
-    // private static final int TOTAL_LEN = 1000 * ARRAY_SIZE ;
-    // just use for load profile and export profile
-    public static final String JOB_ID = "Job ID";
-    public static final String QUERY_ID = "Query ID";
-    public static final String START_TIME = "Start Time";
-    public static final String END_TIME = "End Time";
-    public static final String TOTAL_TIME = "Total";
-    public static final String QUERY_TYPE = "Query Type";
-    public static final String QUERY_STATE = "Query State";
-    public static final String DORIS_VERSION = "Doris Version";
-    public static final String USER = "User";
-    public static final String DEFAULT_DB = "Default Db";
-    public static final String SQL_STATEMENT = "Sql Statement";
-    public static final String IS_CACHED = "Is Cached";
-
-    public static final String TOTAL_INSTANCES_NUM = "Total Instances Num";
-
-    public static final String INSTANCES_NUM_PER_BE = "Instances Num Per BE";
-
-    public static final String TRACE_ID = "Trace ID";
 
     public enum ProfileType {
         QUERY,
         LOAD,
     }
 
-    public static final List<String> PROFILE_HEADERS = Collections.unmodifiableList(
-            Arrays.asList(JOB_ID, QUERY_ID, USER, DEFAULT_DB, SQL_STATEMENT, QUERY_TYPE,
-                    START_TIME, END_TIME, TOTAL_TIME, QUERY_STATE, TRACE_ID));
-
-    private class ProfileElement {
+    public static class ProfileElement {
         public ProfileElement(RuntimeProfile profile) {
             this.profile = profile;
         }
 
         private final RuntimeProfile profile;
-        // cache the result of getProfileContent method
-        private volatile String profileContent;
         public Map<String, String> infoStrings = Maps.newHashMap();
         public MultiProfileTreeBuilder builder = null;
         public String errMsg = "";
 
+        public StatsErrorEstimator statsErrorEstimator;
+
         // lazy load profileContent because sometimes profileContent is very large
         public String getProfileContent() {
-            if (profileContent != null) {
-                return profileContent;
-            }
             // no need to lock because the possibility of concurrent read is very low
-            profileContent = profile.toString();
-            return profileContent;
+            return profile.toString();
+        }
+
+        public String getProfileBrief() {
+            Gson gson = new GsonBuilder().setPrettyPrinting().create();
+            return gson.toJson(profile.toBrief());
+        }
+
+        public double getError() {
+            return statsErrorEstimator.getQError();
+        }
+
+        public void setStatsErrorEstimator(StatsErrorEstimator statsErrorEstimator) {
+            this.statsErrorEstimator = statsErrorEstimator;
         }
     }
 
-    // only protect queryIdDeque; queryIdToProfileMap is concurrent, no need to protect
+    // only protect queryIdDeque; queryIdToProfileMap is concurrent, no need to
+    // protect
     private ReentrantReadWriteLock lock;
     private ReadLock readLock;
     private WriteLock writeLock;
@@ -141,9 +129,16 @@ public class ProfileManager {
     public ProfileElement createElement(RuntimeProfile profile) {
         ProfileElement element = new ProfileElement(profile);
         RuntimeProfile summaryProfile = profile.getChildList().get(0).first;
-        for (String header : PROFILE_HEADERS) {
+        for (String header : SummaryProfile.SUMMARY_KEYS) {
             element.infoStrings.put(header, summaryProfile.getInfoString(header));
         }
+        List<Pair<RuntimeProfile, Boolean>> childList = summaryProfile.getChildList();
+        if (!childList.isEmpty()) {
+            RuntimeProfile executionProfile = childList.get(0).first;
+            for (String header : SummaryProfile.EXECUTION_SUMMARY_KEYS) {
+                element.infoStrings.put(header, executionProfile.getInfoString(header));
+            }
+        }
 
         MultiProfileTreeBuilder builder = new MultiProfileTreeBuilder(profile);
         try {
@@ -164,18 +159,19 @@ public class ProfileManager {
 
         ProfileElement element = createElement(profile);
         // 'insert into' does have job_id, put all profiles key with query_id
-        String key = element.infoStrings.get(ProfileManager.QUERY_ID);
-        // check when push in, which can ensure every element in the list has QUERY_ID column,
+        String key = element.infoStrings.get(SummaryProfile.PROFILE_ID);
+        // check when push in, which can ensure every element in the list has QUERY_ID
+        // column,
         // so there is no need to check when remove element from list.
         if (Strings.isNullOrEmpty(key)) {
             LOG.warn("the key or value of Map is null, "
                     + "may be forget to insert 'QUERY_ID' or 'JOB_ID' column into infoStrings");
         }
 
+        writeLock.lock();
         // a profile may be updated multiple times in queryIdToProfileMap,
         // and only needs to be inserted into the queryIdDeque for the first time.
         queryIdToProfileMap.put(key, element);
-        writeLock.lock();
         try {
             if (!queryIdDeque.contains(key)) {
                 if (queryIdDeque.size() >= Config.max_query_profile_num) {
@@ -199,18 +195,18 @@ public class ProfileManager {
         try {
             Iterator reverse = queryIdDeque.descendingIterator();
             while (reverse.hasNext()) {
-                String  queryId = (String) reverse.next();
+                String queryId = (String) reverse.next();
                 ProfileElement profileElement = queryIdToProfileMap.get(queryId);
                 if (profileElement == null) {
                     continue;
                 }
                 Map<String, String> infoStrings = profileElement.infoStrings;
-                if (type != null && !infoStrings.get(QUERY_TYPE).equalsIgnoreCase(type.name())) {
+                if (type != null && !infoStrings.get(SummaryProfile.TASK_TYPE).equalsIgnoreCase(type.name())) {
                     continue;
                 }
 
                 List<String> row = Lists.newArrayList();
-                for (String str : PROFILE_HEADERS) {
+                for (String str : SummaryProfile.SUMMARY_KEYS) {
                     row.add(infoStrings.get(str));
                 }
                 result.add(row);
@@ -234,6 +230,23 @@ public class ProfileManager {
         }
     }
 
+    public String getProfileBrief(String queryID) {
+        readLock.lock();
+        try {
+            ProfileElement element = queryIdToProfileMap.get(queryID);
+            if (element == null) {
+                return null;
+            }
+            return element.getProfileBrief();
+        } finally {
+            readLock.unlock();
+        }
+    }
+
+    public ProfileElement findProfileElementObject(String queryId) {
+        return queryIdToProfileMap.get(queryId);
+    }
+
     /**
      * Check if the query with specific query id is queried by specific user.
      *
@@ -248,7 +261,7 @@ public class ProfileManager {
             if (element == null) {
                 throw new AuthenticationException("query with id " + queryId + " not found");
             }
-            if (!element.infoStrings.get(USER).equals(user)) {
+            if (!element.infoStrings.get(SummaryProfile.USER).equals(user)) {
                 throw new AuthenticationException("Access deny to view query with id: " + queryId);
             }
         } finally {
@@ -340,7 +353,7 @@ public class ProfileManager {
         readLock.lock();
         try {
             for (Map.Entry<String, ProfileElement> entry : queryIdToProfileMap.entrySet()) {
-                if (entry.getValue().infoStrings.getOrDefault(TRACE_ID, "").equals(traceId)) {
+                if (entry.getValue().infoStrings.getOrDefault(SummaryProfile.TRACE_ID, "").equals(traceId)) {
                     return entry.getKey();
                 }
             }
@@ -349,4 +362,21 @@ public class ProfileManager {
             readLock.unlock();
         }
     }
+
+    public void setStatsErrorEstimator(String queryId, StatsErrorEstimator statsErrorEstimator) {
+        ProfileElement profileElement = findProfileElementObject(queryId);
+        if (profileElement != null) {
+            profileElement.setStatsErrorEstimator(statsErrorEstimator);
+        }
+    }
+
+    public void cleanProfile() {
+        writeLock.lock();
+        try {
+            queryIdToProfileMap.clear();
+            queryIdDeque.clear();
+        } finally {
+            writeLock.unlock();
+        }
+    }
 }
