@@ -153,6 +153,9 @@ public interface CatalogIf<T extends DatabaseIf> {
 
     String getComment();
 
+    default void setComment(String comment) {
+    }
+
     default long getLastUpdateTime() {
         return -1L;
     }
@@ -172,4 +175,6 @@ public interface CatalogIf<T extends DatabaseIf> {
     public Collection<DatabaseIf> getAllDbs();
 
     public ConcurrentHashMap<Long, DatabaseIf> getIdToDb();
+
+    public boolean enableAutoAnalyze();
 }
