@@ -19,6 +19,7 @@ package org.apache.doris.datasource.jdbc;
 
 import org.apache.doris.catalog.JdbcResource;
 import org.apache.doris.catalog.external.JdbcExternalDatabase;
+import org.apache.doris.common.Config;
 import org.apache.doris.common.DdlException;
 import org.apache.doris.datasource.CatalogProperty;
 import org.apache.doris.datasource.ExternalCatalog;
@@ -28,6 +29,7 @@ import org.apache.doris.datasource.jdbc.client.JdbcClient;
 import org.apache.doris.datasource.jdbc.client.JdbcClientConfig;
 
 import com.google.common.base.Strings;
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import lombok.Getter;
@@ -38,7 +40,7 @@ import java.util.Map;
 
 @Getter
 public class JdbcExternalCatalog extends ExternalCatalog {
-    private static final List<String> REQUIRED_PROPERTIES = Lists.newArrayList(
+    private static final List<String> REQUIRED_PROPERTIES = ImmutableList.of(
             JdbcResource.JDBC_URL,
             JdbcResource.DRIVER_URL,
             JdbcResource.DRIVER_CLASS
@@ -124,6 +126,12 @@ public class JdbcExternalCatalog extends ExternalCatalog {
     }
 
     public String getLowerCaseTableNames() {
+        // Forced to true if Config.lower_case_table_names has a value of 1 or 2
+        if (Config.lower_case_table_names == 1 || Config.lower_case_table_names == 2) {
+            return "true";
+        }
+
+        // Otherwise, it defaults to false
         return catalogProperty.getOrDefault(JdbcResource.LOWER_CASE_TABLE_NAMES, "false");
     }
 
