@@ -198,8 +198,7 @@ public class ShowAction extends RestBaseController {
         } else {
             for (long dbId : Env.getCurrentInternalCatalog().getDbIds()) {
                 DatabaseIf db = Env.getCurrentInternalCatalog().getDbNullable(dbId);
-                if (db == null || !(db instanceof Database) || ((Database) db).isInfoSchemaDb()
-                        || ((Database) db).isMysqlDb()) {
+                if (db == null || !(db instanceof Database) || ((Database) db).isMysqlCompatibleDatabase()) {
                     continue;
                 }
                 totalSize += getDataSizeOfDatabase(db);
