@@ -22,6 +22,7 @@ import org.apache.doris.analysis.TablePattern;
 import org.apache.doris.analysis.UserIdentity;
 import org.apache.doris.analysis.WorkloadGroupPattern;
 import org.apache.doris.catalog.Env;
+import org.apache.doris.cluster.ClusterNamespace;
 import org.apache.doris.common.AnalysisException;
 import org.apache.doris.common.DdlException;
 import org.apache.doris.common.FeMetaVersion;
@@ -37,14 +38,18 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
 import com.google.gson.annotations.SerializedName;
+import org.apache.commons.collections.CollectionUtils;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
 import java.io.DataInput;
 import java.io.DataOutput;
 import java.io.IOException;
+import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Objects;
+import java.util.Optional;
 import java.util.Set;
 
 public class Role implements Writable, GsonPostProcessable {
@@ -80,6 +85,8 @@ public class Role implements Writable, GsonPostProcessable {
     private Map<ResourcePattern, PrivBitSet> resourcePatternToPrivs = Maps.newConcurrentMap();
     @SerializedName(value = "workloadGroupPatternToPrivs")
     private Map<WorkloadGroupPattern, PrivBitSet> workloadGroupPatternToPrivs = Maps.newConcurrentMap();
+    @SerializedName(value = "colPrivMap")
+    private Map<ColPrivilegeKey, Set<String>> colPrivMap = Maps.newHashMap();
 
     // Will not be persisted, generated by tblPatternToPrivs and resourcePatternToPrivs
     private GlobalPrivTable globalPrivTable = new GlobalPrivTable();
@@ -89,6 +96,7 @@ public class Role implements Writable, GsonPostProcessable {
     private ResourcePrivTable resourcePrivTable = new ResourcePrivTable();
     private WorkloadGroupPrivTable workloadGroupPrivTable = new WorkloadGroupPrivTable();
 
+
     @Deprecated
     private Set<UserIdentity> users = Sets.newConcurrentHashSet();
 
@@ -111,6 +119,14 @@ public class Role implements Writable, GsonPostProcessable {
         grantPrivs(tablePattern, privs.copy());
     }
 
+    public Role(String roleName, TablePattern tablePattern, PrivBitSet privs,
+            Map<ColPrivilegeKey, Set<String>> colPrivileges) throws DdlException {
+        this.roleName = roleName;
+        this.tblPatternToPrivs.put(tablePattern, privs);
+        grantPrivs(tablePattern, privs.copy());
+        grantCols(colPrivileges);
+    }
+
     public Role(String roleName, ResourcePattern resourcePattern, PrivBitSet privs) throws DdlException {
         this.roleName = roleName;
         this.resourcePatternToPrivs.put(resourcePattern, privs);
@@ -123,17 +139,23 @@ public class Role implements Writable, GsonPostProcessable {
         grantPrivs(workloadGroupPattern, privs.copy());
     }
 
-    public Role(String roleName, TablePattern tablePattern, PrivBitSet tablePrivs,
+    public Role(String roleName, List<TablePattern> tablePatterns, PrivBitSet tablePrivs,
             WorkloadGroupPattern workloadGroupPattern, PrivBitSet workloadGroupPrivs) {
         this.roleName = roleName;
-        this.tblPatternToPrivs.put(tablePattern, tablePrivs);
         this.workloadGroupPatternToPrivs.put(workloadGroupPattern, workloadGroupPrivs);
-        //for init admin role,will not generate exception
+        tablePatterns.forEach(tablePattern -> {
+            // for init admin role,will not generate exception
+            try {
+                this.tblPatternToPrivs.put(tablePattern, tablePrivs);
+                grantPrivs(tablePattern, tablePrivs.copy());
+            } catch (DdlException e) {
+                LOG.warn("grant table failed,", e);
+            }
+        });
         try {
-            grantPrivs(tablePattern, tablePrivs.copy());
             grantPrivs(workloadGroupPattern, workloadGroupPrivs.copy());
         } catch (DdlException e) {
-            LOG.warn("grant failed,", e);
+            LOG.warn("grant workload group failed,", e);
         }
     }
 
@@ -186,6 +208,18 @@ public class Role implements Writable, GsonPostProcessable {
             }
             grantPrivs(entry.getKey(), entry.getValue().copy());
         }
+        mergeColPrivMap(colPrivMap, other.colPrivMap);
+    }
+
+    public static void mergeColPrivMap(Map<ColPrivilegeKey, Set<String>> toColPrivMap,
+            Map<ColPrivilegeKey, Set<String>> fromColPrivMap) {
+        for (Entry<ColPrivilegeKey, Set<String>> entry : fromColPrivMap.entrySet()) {
+            if (toColPrivMap.containsKey(entry.getKey())) {
+                toColPrivMap.get(entry.getKey()).addAll(entry.getValue());
+            } else {
+                toColPrivMap.put(entry.getKey(), entry.getValue());
+            }
+        }
     }
 
     public void merge(Role other) throws DdlException {
@@ -227,7 +261,8 @@ public class Role implements Writable, GsonPostProcessable {
      */
     private boolean checkAnyPrivWithinCatalog(String ctl) {
         return dbPrivTable.hasPrivsOfCatalog(ctl)
-                || tablePrivTable.hasPrivsOfCatalog(ctl);
+                || tablePrivTable.hasPrivsOfCatalog(ctl)
+                || checkAnyColPrivWithinCtl(ctl);
 
     }
 
@@ -263,10 +298,44 @@ public class Role implements Writable, GsonPostProcessable {
      * if so, the database should be visible to this user.
      */
     private boolean checkAnyPrivWithinDb(String ctl, String db) {
-        return tablePrivTable.hasPrivsOfDb(ctl, db);
+        return tablePrivTable.hasPrivsOfDb(ctl, db) || checkAnyColPrivWithinDb(ctl, db);
 
     }
 
+    private boolean checkAnyColPrivWithinCtl(String ctl) {
+        ctl = ClusterNamespace.getNameFromFullName(ctl);
+        for (ColPrivilegeKey colPrivilegeKey : colPrivMap.keySet()) {
+            if (colPrivilegeKey.getCtl().equals(ctl)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    private boolean checkAnyColPrivWithinDb(String ctl, String db) {
+        ctl = ClusterNamespace.getNameFromFullName(ctl);
+        db = ClusterNamespace.getNameFromFullName(db);
+        for (ColPrivilegeKey colPrivilegeKey : colPrivMap.keySet()) {
+            if (colPrivilegeKey.getCtl().equals(ctl) && colPrivilegeKey.getDb().equals(db)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    private boolean checkAnyColPrivWithinTbl(String ctl, String db, String tbl) {
+        ctl = ClusterNamespace.getNameFromFullName(ctl);
+        db = ClusterNamespace.getNameFromFullName(db);
+        tbl = ClusterNamespace.getNameFromFullName(tbl);
+        for (ColPrivilegeKey colPrivilegeKey : colPrivMap.keySet()) {
+            if (colPrivilegeKey.getCtl().equals(ctl) && colPrivilegeKey.getDb().equals(db) && colPrivilegeKey
+                    .getTbl().equals(tbl)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     private boolean checkDbInternal(String ctl, String db, PrivPredicate wanted,
             PrivBitSet savedPrivs) {
         dbPrivTable.getPrivs(ctl, db, savedPrivs);
@@ -284,10 +353,32 @@ public class Role implements Writable, GsonPostProcessable {
                 || checkTblInternal(ctl, db, tbl, wanted, savedPrivs)) {
             return true;
         }
+
+        // if user has any privs of col in this table, and the wanted priv is SHOW, return true
+        if (ctl != null && db != null && tbl != null && wanted == PrivPredicate.SHOW && checkAnyColPrivWithinTbl(ctl,
+                db, tbl)) {
+            return true;
+        }
+
         LOG.debug("failed to get wanted privs: {}, granted: {}", wanted, savedPrivs);
         return false;
     }
 
+    public boolean checkColPriv(String ctl, String db, String tbl, String col, PrivPredicate wanted) {
+        Optional<Privilege> colPrivilege = wanted.getColPrivilege();
+        Preconditions.checkState(colPrivilege.isPresent(), "this privPredicate should not use checkColPriv:" + wanted);
+        return checkTblPriv(ctl, db, tbl, wanted) || onlyCheckColPriv(ctl, db, tbl, col, colPrivilege.get());
+    }
+
+    private boolean onlyCheckColPriv(String ctl, String db, String tbl, String col,
+            Privilege colPrivilege) {
+        ColPrivilegeKey colPrivilegeKey = new ColPrivilegeKey(colPrivilege, ctl, db, tbl);
+        if (colPrivMap.containsKey(colPrivilegeKey)) {
+            return colPrivMap.get(colPrivilegeKey).contains(col);
+        }
+        return false;
+    }
+
     private boolean checkTblInternal(String ctl, String db, String tbl, PrivPredicate wanted, PrivBitSet savedPrivs) {
         tablePrivTable.getPrivs(ctl, db, tbl, savedPrivs);
         return Privilege.satisfy(savedPrivs, wanted);
@@ -371,6 +462,10 @@ public class Role implements Writable, GsonPostProcessable {
         return tablePrivTable;
     }
 
+    public Map<ColPrivilegeKey, Set<String>> getColPrivMap() {
+        return colPrivMap;
+    }
+
     public ResourcePrivTable getResourcePrivTable() {
         return resourcePrivTable;
     }
@@ -396,7 +491,9 @@ public class Role implements Writable, GsonPostProcessable {
 
 
     private void grantPrivs(ResourcePattern resourcePattern, PrivBitSet privs) throws DdlException {
-
+        if (privs.isEmpty()) {
+            return;
+        }
         // grant privs to user
         switch (resourcePattern.getPrivLevel()) {
             case GLOBAL:
@@ -412,12 +509,31 @@ public class Role implements Writable, GsonPostProcessable {
     }
 
     private void grantPrivs(WorkloadGroupPattern workloadGroupPattern, PrivBitSet privs) throws DdlException {
+        if (privs.isEmpty()) {
+            return;
+        }
         if (workloadGroupPattern.getPrivLevel().equals(PrivLevel.WORKLOAD_GROUP)) {
             grantWorkloadGroupPrivs(workloadGroupPattern.getworkloadGroupName(), privs);
         }
     }
 
+    private void grantCols(Map<ColPrivilegeKey, Set<String>> colPrivileges) {
+        if (Objects.isNull(colPrivileges)) {
+            return;
+        }
+        for (Entry<ColPrivilegeKey, Set<String>> entry : colPrivileges.entrySet()) {
+            if (colPrivMap.containsKey(entry.getKey())) {
+                colPrivMap.get(entry.getKey()).addAll(entry.getValue());
+            } else {
+                colPrivMap.put(entry.getKey(), Sets.newHashSet(entry.getValue()));
+            }
+        }
+    }
+
     private void grantPrivs(TablePattern tblPattern, PrivBitSet privs) throws DdlException {
+        if (privs.isEmpty()) {
+            return;
+        }
         // grant privs to user
         switch (tblPattern.getPrivLevel()) {
             case GLOBAL:
@@ -501,7 +617,9 @@ public class Role implements Writable, GsonPostProcessable {
         workloadGroupPrivTable.addEntry(entry, false, false);
     }
 
-    public void revokePrivs(TablePattern tblPattern, PrivBitSet privs, boolean errOnNonExist) throws DdlException {
+    public void revokePrivs(TablePattern tblPattern, PrivBitSet privs, Map<ColPrivilegeKey, Set<String>> colPrivileges,
+            boolean errOnNonExist)
+            throws DdlException {
         PrivBitSet existingPriv = tblPatternToPrivs.get(tblPattern);
         if (existingPriv == null) {
             if (errOnNonExist) {
@@ -511,6 +629,21 @@ public class Role implements Writable, GsonPostProcessable {
         }
         existingPriv.remove(privs);
         revokePrivs(tblPattern, privs);
+        revokeCols(colPrivileges);
+    }
+
+    private void revokeCols(Map<ColPrivilegeKey, Set<String>> colPrivileges) {
+        if (Objects.isNull(colPrivileges)) {
+            return;
+        }
+        for (Entry<ColPrivilegeKey, Set<String>> entry : colPrivileges.entrySet()) {
+            if (colPrivMap.containsKey(entry.getKey())) {
+                colPrivMap.get(entry.getKey()).removeAll(entry.getValue());
+                if (CollectionUtils.isEmpty(colPrivMap.get(entry.getKey()))) {
+                    colPrivMap.remove(entry.getKey());
+                }
+            }
+        }
     }
 
     public void revokePrivs(ResourcePattern resourcePattern, PrivBitSet privs, boolean errOnNonExist)
