@@ -62,7 +62,9 @@ public class Optimizer {
                 || maxJoinCount > getSessionVariable().getMaxTableCountUseCascadesJoinReorder();
         cascadesContext.getStatementContext().setDpHyp(isDpHyp);
         cascadesContext.getStatementContext().setOtherJoinReorder(false);
-        if (!getSessionVariable().isDisableJoinReorder() && isDpHyp) {
+        if (!getSessionVariable().isDisableJoinReorder() && isDpHyp
+                && maxJoinCount <= getSessionVariable().getMaxJoinNumberOfReorder()) {
+            //RightNow, dphyper can only order 64 join operators
             dpHypOptimize();
         }
 
@@ -78,8 +80,6 @@ public class Optimizer {
         // Due to EnsureProjectOnTopJoin, root group can't be Join Group, so DPHyp doesn't change the root group
         cascadesContext.pushJob(new JoinOrderJob(root, cascadesContext.getCurrentJobContext()));
         cascadesContext.getJobScheduler().executeJobPool(cascadesContext);
-        // after DPHyp just keep logical expression
-        cascadesContext.getMemo().removePhysicalExpression();
         cascadesContext.getStatementContext().setOtherJoinReorder(true);
     }
 
