@@ -23,6 +23,7 @@ import org.apache.doris.nereids.trees.expressions.Alias;
 import org.apache.doris.nereids.trees.expressions.ExprId;
 import org.apache.doris.nereids.trees.expressions.NamedExpression;
 import org.apache.doris.nereids.trees.expressions.Slot;
+import org.apache.doris.nereids.trees.expressions.SlotReference;
 import org.apache.doris.nereids.trees.plans.JoinType;
 import org.apache.doris.nereids.trees.plans.Plan;
 import org.apache.doris.nereids.trees.plans.algebra.EmptyRelation;
@@ -74,21 +75,27 @@ public class EliminateEmptyRelation implements RewriteRuleFactory {
 
             // after BuildAggForUnion rule, union may have more than 2 children.
             logicalUnion(multi()).then(union -> {
-                if (union.children().size() == 0) {
+                if (union.children().isEmpty()) {
                     // example: select * from (select 1,2 union select 3, 4) T;
                     // the children size is 0. (1,2) and (3,4) are stored in union.constantExprsList
                     return null;
                 }
-                List<Plan> nonEmptyChildren = union.children().stream()
-                        .filter(child -> !(child instanceof EmptyRelation))
-                        .collect(Collectors.toList());
+                ImmutableList.Builder<Plan> nonEmptyChildrenBuilder = ImmutableList.builder();
+                ImmutableList.Builder<List<SlotReference>> nonEmptyOutputsBuilder = ImmutableList.builder();
+                for (int i = 0; i < union.arity(); i++) {
+                    if (!(union.child(i) instanceof EmptyRelation)) {
+                        nonEmptyChildrenBuilder.add(union.child(i));
+                        nonEmptyOutputsBuilder.add(union.getRegularChildOutput(i));
+                    }
+                }
+                List<Plan> nonEmptyChildren = nonEmptyChildrenBuilder.build();
                 if (nonEmptyChildren.isEmpty()) {
                     if (union.getConstantExprsList().isEmpty()) {
                         return new LogicalEmptyRelation(
                                 ConnectContext.get().getStatementContext().getNextRelationId(),
                                 union.getOutput());
                     } else {
-                        return union.withChildren(ImmutableList.of());
+                        return union.withChildrenAndTheirOutputs(ImmutableList.of(), ImmutableList.of());
                     }
                 } else if (nonEmptyChildren.size() == 1) {
                     if (union.getConstantExprsList().isEmpty()) {
@@ -102,13 +109,12 @@ public class EliminateEmptyRelation implements RewriteRuleFactory {
                             projects.add(alias);
                         }
 
-                        LogicalProject project = new LogicalProject<>(projects, child);
-                        return project;
+                        return new LogicalProject<>(projects, child);
                     }
                 }
 
                 if (union.children().size() != nonEmptyChildren.size()) {
-                    return union.withChildren(ImmutableList.copyOf(nonEmptyChildren));
+                    return union.withChildrenAndTheirOutputs(nonEmptyChildren, nonEmptyOutputsBuilder.build());
                 } else {
                     // no empty relation child, do not change union
                     return null;
@@ -138,9 +144,15 @@ public class EliminateEmptyRelation implements RewriteRuleFactory {
                             ConnectContext.get().getStatementContext().getNextRelationId(),
                             except.getOutput());
                 } else {
-                    List<Plan> nonEmptyChildren = except.children().stream()
-                            .filter(child -> !(child instanceof EmptyRelation))
-                            .collect(Collectors.toList());
+                    ImmutableList.Builder<Plan> nonEmptyChildrenBuilder = ImmutableList.builder();
+                    ImmutableList.Builder<List<SlotReference>> nonEmptyOutputsBuilder = ImmutableList.builder();
+                    for (int i = 0; i < except.arity(); i++) {
+                        if (!(except.child(i) instanceof EmptyRelation)) {
+                            nonEmptyChildrenBuilder.add(except.child(i));
+                            nonEmptyOutputsBuilder.add(except.getRegularChildOutput(i));
+                        }
+                    }
+                    List<Plan> nonEmptyChildren = nonEmptyChildrenBuilder.build();
                     if (nonEmptyChildren.size() == 1) {
                         // the first child is not empty, others are all empty
                         // case 1. FIRST except(distinct) empty = > project(AGG(FIRST))
@@ -155,22 +167,19 @@ public class EliminateEmptyRelation implements RewriteRuleFactory {
                         } else {
                             projectChild = first;
                         }
-
                         List<Slot> exceptOutput = except.getOutput();
                         List<Slot> projectInputSlots = projectChild.getOutput();
-
                         List<NamedExpression> projects = Lists.newArrayList();
                         for (int i = 0; i < exceptOutput.size(); i++) {
                             ExprId id = exceptOutput.get(i).getExprId();
                             Alias alias = new Alias(id, projectInputSlots.get(i), exceptOutput.get(i).getName());
                             projects.add(alias);
                         }
-                        LogicalProject project = new LogicalProject(projects, projectChild);
-                        return project;
+                        return new LogicalProject<>(projects, projectChild);
                     } else if (nonEmptyChildren.size() == except.children().size()) {
                         return null;
                     } else {
-                        return except.withChildren(nonEmptyChildren);
+                        return except.withChildrenAndTheirOutputs(nonEmptyChildren, nonEmptyOutputsBuilder.build());
                     }
                 }
             }).toRule(RuleType.ELIMINATE_EXCEPT_ON_EMPTYRELATION)
