@@ -30,6 +30,8 @@ import org.apache.doris.nereids.util.TypeCoercionUtils;
 
 import com.google.common.base.Preconditions;
 
+import java.util.List;
+
 /**
  * binary arithmetic operator. Such as +, -, *, /.
  */
@@ -37,8 +39,8 @@ public abstract class BinaryArithmetic extends BinaryOperator implements Propaga
 
     private final Operator legacyOperator;
 
-    public BinaryArithmetic(Expression left, Expression right, Operator legacyOperator) {
-        super(left, right, legacyOperator.toString());
+    public BinaryArithmetic(List<Expression> children, Operator legacyOperator) {
+        super(children, legacyOperator.toString());
         this.legacyOperator = legacyOperator;
     }
 
@@ -59,7 +61,11 @@ public abstract class BinaryArithmetic extends BinaryOperator implements Propaga
             return getDataTypeForDecimalV2((DecimalV2Type) t1, (DecimalV2Type) t2);
         }
         if (t1.isDecimalV3Type() && t2.isDecimalV3Type()) {
-            return getDataTypeForDecimalV3((DecimalV3Type) t1, (DecimalV3Type) t2);
+            if (this instanceof Add || this instanceof Subtract || this instanceof Mod) {
+                return t1;
+            } else {
+                return getDataTypeForDecimalV3((DecimalV3Type) t1, (DecimalV3Type) t2);
+            }
         }
         return getDataTypeForOthers(t1, t2);
     }
