@@ -23,8 +23,8 @@ import org.apache.doris.nereids.exceptions.AnalysisException;
 import org.apache.doris.nereids.exceptions.UnboundException;
 import org.apache.doris.nereids.properties.PhysicalProperties;
 import org.apache.doris.nereids.trees.expressions.Expression;
+import org.apache.doris.nereids.trees.expressions.Properties;
 import org.apache.doris.nereids.trees.expressions.Slot;
-import org.apache.doris.nereids.trees.expressions.TVFProperties;
 import org.apache.doris.nereids.trees.expressions.functions.BoundFunction;
 import org.apache.doris.nereids.trees.expressions.functions.CustomSignature;
 import org.apache.doris.nereids.trees.expressions.shape.UnaryExpression;
@@ -57,7 +57,7 @@ public abstract class TableValuedFunction extends BoundFunction implements Unary
         }
     });
 
-    public TableValuedFunction(String functionName, TVFProperties tvfProperties) {
+    public TableValuedFunction(String functionName, Properties tvfProperties) {
         super(functionName, tvfProperties);
     }
 
@@ -78,8 +78,8 @@ public abstract class TableValuedFunction extends BoundFunction implements Unary
         return new Statistics(0, columnToStatistics);
     }
 
-    public TVFProperties getTVFProperties() {
-        return (TVFProperties) child(0);
+    public Properties getTVFProperties() {
+        return (Properties) child(0);
     }
 
     public final String getTableName() {
