@@ -197,7 +197,7 @@ public final class QeProcessorImpl implements QeProcessor {
             } else {
                 result.setStatus(new TStatus(TStatusCode.RUNTIME_ERROR));
             }
-            LOG.info("ReportExecStatus() runtime error, query {} with type {} does not exist",
+            LOG.warn("ReportExecStatus() runtime error, query {} with type {} does not exist",
                     DebugUtil.printId(params.query_id), params.query_type);
             return result;
         }
