@@ -25,14 +25,14 @@ import org.apache.doris.common.UserException;
 import org.apache.doris.common.util.S3URI;
 import org.apache.doris.datasource.credentials.CloudCredentialWithEndpoint;
 import org.apache.doris.datasource.property.PropertyConverter;
+import org.apache.doris.datasource.property.S3ClientBEProperties;
 import org.apache.doris.datasource.property.constants.S3Properties;
 import org.apache.doris.fs.FileSystemFactory;
 import org.apache.doris.thrift.TFileType;
 
-import com.google.common.base.Preconditions;
+import com.google.common.base.Strings;
 import com.google.common.collect.ImmutableSet;
 
-import java.util.HashMap;
 import java.util.Map;
 
 /**
@@ -48,70 +48,46 @@ import java.util.Map;
  */
 public class S3TableValuedFunction extends ExternalFileTableValuedFunction {
     public static final String NAME = "s3";
-    public static final String S3_URI = "uri";
+    public static final String PROP_URI = "uri";
 
     private static final ImmutableSet<String> DEPRECATED_KEYS =
-            ImmutableSet.of("access_key", "secret_key", "session_token", "region");
+            ImmutableSet.of("access_key", "secret_key", "session_token", "region",
+                    "ACCESS_KEY", "SECRET_KEY", "SESSION_TOKEN", "REGION");
 
-    private static final ImmutableSet<String> OPTIONAL_KEYS =
-            ImmutableSet.of(S3Properties.SESSION_TOKEN, PropertyConverter.USE_PATH_STYLE, S3Properties.REGION,
-                    PATH_PARTITION_KEYS);
-
-    private static final ImmutableSet<String> LOCATION_PROPERTIES = ImmutableSet.<String>builder()
-            .add(S3_URI)
-            .add(S3Properties.ENDPOINT)
-            .addAll(DEPRECATED_KEYS)
-            .addAll(S3Properties.TVF_REQUIRED_FIELDS)
-            .addAll(OPTIONAL_KEYS)
-            .build();
-
-    private final S3URI s3uri;
-    private final boolean forceVirtualHosted;
     private String virtualBucket = "";
 
-    public S3TableValuedFunction(Map<String, String> params) throws AnalysisException {
+    public S3TableValuedFunction(Map<String, String> properties) throws AnalysisException {
+        // 1. analyze common properties
+        Map<String, String> otherProps = super.parseCommonProperties(properties);
 
-        Map<String, String> fileParams = new HashMap<>();
-        for (Map.Entry<String, String> entry : params.entrySet()) {
-            String key = entry.getKey();
-            String lowerKey = key.toLowerCase();
-            if (!LOCATION_PROPERTIES.contains(lowerKey) && !FILE_FORMAT_PROPERTIES.contains(lowerKey)) {
-                throw new AnalysisException("Invalid property: " + key);
-            }
-            if (DEPRECATED_KEYS.contains(lowerKey)) {
-                lowerKey = S3Properties.S3_PREFIX + lowerKey;
-            }
-            fileParams.put(lowerKey, entry.getValue());
+        // 2. analyze uri and other properties
+        String uriStr = getOrDefaultAndRemove(otherProps, PROP_URI, null);
+        if (Strings.isNullOrEmpty(uriStr)) {
+            throw new AnalysisException(String.format("Properties '%s' is required.", PROP_URI));
         }
+        forwardCompatibleDeprecatedKeys(otherProps);
 
-        if (!fileParams.containsKey(S3_URI)) {
-            throw new AnalysisException("Missing required property: " + S3_URI);
-        }
-
-        forceVirtualHosted = isVirtualHosted(fileParams);
-        s3uri = getS3Uri(fileParams);
-        final String endpoint = forceVirtualHosted
-                ? getEndpointAndSetVirtualBucket(params)
-                : s3uri.getBucketScheme();
-        if (!fileParams.containsKey(S3Properties.REGION)) {
+        String usePathStyle = getOrDefaultAndRemove(otherProps, PropertyConverter.USE_PATH_STYLE, "false");
+        boolean forceVirtualHosted = isVirtualHosted(uriStr, Boolean.parseBoolean(usePathStyle));
+        S3URI s3uri = getS3Uri(uriStr, forceVirtualHosted);
+        String endpoint = forceVirtualHosted
+                ? getEndpointAndSetVirtualBucket(s3uri, otherProps) : s3uri.getBucketScheme();
+        if (!otherProps.containsKey(S3Properties.REGION)) {
             String region = S3Properties.getRegionOfEndpoint(endpoint);
-            fileParams.put(S3Properties.REGION, region);
+            otherProps.put(S3Properties.REGION, region);
         }
+        checkNecessaryS3Properties(otherProps);
         CloudCredentialWithEndpoint credential = new CloudCredentialWithEndpoint(endpoint,
-                fileParams.get(S3Properties.REGION),
-                fileParams.get(S3Properties.ACCESS_KEY),
-                fileParams.get(S3Properties.SECRET_KEY));
-        if (fileParams.containsKey(S3Properties.SESSION_TOKEN)) {
-            credential.setSessionToken(fileParams.get(S3Properties.SESSION_TOKEN));
+                otherProps.get(S3Properties.REGION),
+                otherProps.get(S3Properties.ACCESS_KEY),
+                otherProps.get(S3Properties.SECRET_KEY));
+        if (otherProps.containsKey(S3Properties.SESSION_TOKEN)) {
+            credential.setSessionToken(otherProps.get(S3Properties.SESSION_TOKEN));
         }
 
-        // set S3 location properties
-        // these five properties is necessary, no one can be lost.
         locationProperties = S3Properties.credentialToMap(credential);
-        String usePathStyle = fileParams.getOrDefault(PropertyConverter.USE_PATH_STYLE, "false");
         locationProperties.put(PropertyConverter.USE_PATH_STYLE, usePathStyle);
-
-        super.parseProperties(fileParams);
+        locationProperties.putAll(S3ClientBEProperties.getBeFSProperties(locationProperties));
 
         if (forceVirtualHosted) {
             filePath = NAME + S3URI.SCHEME_DELIM + virtualBucket + S3URI.PATH_DELIM
@@ -128,39 +104,59 @@ public class S3TableValuedFunction extends ExternalFileTableValuedFunction {
         }
     }
 
-    private String getEndpointAndSetVirtualBucket(Map<String, String> params) throws AnalysisException {
-        Preconditions.checkState(forceVirtualHosted, "only invoked when force virtual hosted.");
-        String[] fileds = s3uri.getVirtualBucket().split("\\.", 2);
-        virtualBucket = fileds[0];
-        if (fileds.length > 1) {
+    private void forwardCompatibleDeprecatedKeys(Map<String, String> props) {
+        for (String deprecatedKey : DEPRECATED_KEYS) {
+            String value = props.remove(deprecatedKey);
+            if (!Strings.isNullOrEmpty(value)) {
+                props.put("s3." + deprecatedKey.toLowerCase(), value);
+            }
+        }
+    }
+
+    private void checkNecessaryS3Properties(Map<String, String> props) throws AnalysisException {
+        if (Strings.isNullOrEmpty(props.get(S3Properties.REGION))) {
+            throw new AnalysisException(String.format("Properties '%s' is required.", S3Properties.REGION));
+        }
+        if (Strings.isNullOrEmpty(props.get(S3Properties.ACCESS_KEY))) {
+            throw new AnalysisException(String.format("Properties '%s' is required.", S3Properties.ACCESS_KEY));
+        }
+        if (Strings.isNullOrEmpty(props.get(S3Properties.SECRET_KEY))) {
+            throw new AnalysisException(String.format("Properties '%s' is required.", S3Properties.SECRET_KEY));
+        }
+    }
+
+    private String getEndpointAndSetVirtualBucket(S3URI s3uri, Map<String, String> props)
+            throws AnalysisException {
+        String[] fields = s3uri.getVirtualBucket().split("\\.", 2);
+        virtualBucket = fields[0];
+        if (fields.length > 1) {
             // At this point, s3uri.getVirtualBucket() is: virtualBucket.endpoint, Eg:
             //          uri: http://my_bucket.cos.ap-beijing.myqcloud.com/file.txt
             // s3uri.getVirtualBucket() = my_bucket.cos.ap-beijing.myqcloud.com,
             // so we need separate virtualBucket and endpoint.
-            return fileds[1];
-        } else if (params.containsKey(S3Properties.ENDPOINT)) {
-            return params.get(S3Properties.ENDPOINT);
+            return fields[1];
+        } else if (props.containsKey(S3Properties.ENDPOINT)) {
+            return props.get(S3Properties.ENDPOINT);
         } else {
             throw new AnalysisException("can not parse endpoint, please check uri.");
         }
     }
 
-    private boolean isVirtualHosted(Map<String, String> validParams) {
-        String originUri = validParams.getOrDefault(S3_URI, "");
-        if (originUri.toLowerCase().startsWith("s3")) {
+    private boolean isVirtualHosted(String uri, boolean usePathStyle) {
+        if (uri.toLowerCase().startsWith("s3")) {
             // s3 protocol, default virtual-hosted style
             return true;
         } else {
             // not s3 protocol, forceVirtualHosted is determined by USE_PATH_STYLE.
-            return !Boolean.parseBoolean(validParams.get(PropertyConverter.USE_PATH_STYLE));
+            return !usePathStyle;
         }
     }
 
-    private S3URI getS3Uri(Map<String, String> validParams) throws AnalysisException {
+    private S3URI getS3Uri(String uri, boolean forceVirtualHosted) throws AnalysisException {
         try {
-            return S3URI.create(validParams.get(S3_URI), forceVirtualHosted);
+            return S3URI.create(uri, forceVirtualHosted);
         } catch (UserException e) {
-            throw new AnalysisException("parse s3 uri failed, uri = " + validParams.get(S3_URI), e);
+            throw new AnalysisException("parse s3 uri failed, uri = " + uri, e);
         }
     }
 
@@ -187,3 +183,4 @@ public class S3TableValuedFunction extends ExternalFileTableValuedFunction {
         return "S3TableValuedFunction";
     }
 }
+
