@@ -17,18 +17,11 @@
 
 package org.apache.doris.transaction;
 
-import org.apache.doris.catalog.Database;
 import org.apache.doris.catalog.Env;
-import org.apache.doris.catalog.MaterializedIndex;
-import org.apache.doris.catalog.OlapTable;
-import org.apache.doris.catalog.Partition;
-import org.apache.doris.catalog.Replica;
-import org.apache.doris.catalog.Table;
-import org.apache.doris.catalog.Tablet;
-import org.apache.doris.catalog.TabletInvertedIndex;
 import org.apache.doris.common.Config;
 import org.apache.doris.common.util.MasterDaemon;
 import org.apache.doris.metric.MetricRepo;
+import org.apache.doris.system.SystemInfoService;
 import org.apache.doris.task.AgentBatchTask;
 import org.apache.doris.task.AgentTaskExecutor;
 import org.apache.doris.task.AgentTaskQueue;
@@ -36,8 +29,9 @@ import org.apache.doris.task.PublishVersionTask;
 import org.apache.doris.thrift.TPartitionVersionInfo;
 import org.apache.doris.thrift.TTaskType;
 
-import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
+import org.apache.commons.collections.CollectionUtils;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -45,6 +39,7 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.stream.Stream;
 
 public class PublishVersionDaemon extends MasterDaemon {
 
@@ -63,15 +58,6 @@ public class PublishVersionDaemon extends MasterDaemon {
         }
     }
 
-    private boolean isAllBackendsOfUnfinishedTasksDead(List<PublishVersionTask> unfinishedTasks) {
-        for (PublishVersionTask unfinishedTask : unfinishedTasks) {
-            if (Env.getCurrentSystemInfo().checkBackendAlive(unfinishedTask.getBackendId())) {
-                return false;
-            }
-        }
-        return true;
-    }
-
     private void publishVersion() {
         GlobalTransactionMgr globalTransactionMgr = Env.getCurrentGlobalTransactionMgr();
         List<TransactionState> readyTransactionStates = globalTransactionMgr.getReadyToPublishTransactions();
@@ -81,7 +67,8 @@ public class PublishVersionDaemon extends MasterDaemon {
 
         // ATTN, we publish transaction state to all backends including dead backend, if not publish to dead backend
         // then transaction manager will treat it as success
-        List<Long> allBackends = Env.getCurrentSystemInfo().getBackendIds(false);
+        SystemInfoService infoService = Env.getCurrentSystemInfo();
+        List<Long> allBackends = infoService.getAllBackendIds(false);
         if (allBackends.isEmpty()) {
             LOG.warn("some transaction state need to publish, but no backend exists");
             return;
@@ -130,7 +117,7 @@ public class PublishVersionDaemon extends MasterDaemon {
                 batchTask.addTask(task);
                 transactionState.addPublishVersionTask(backendId, task);
             }
-            transactionState.setHasSendTask(true);
+            transactionState.setSendedTask();
             LOG.info("send publish tasks for transaction: {}, db: {}", transactionState.getTransactionId(),
                     transactionState.getDbId());
         }
@@ -138,109 +125,34 @@ public class PublishVersionDaemon extends MasterDaemon {
             AgentTaskExecutor.submit(batchTask);
         }
 
-        TabletInvertedIndex tabletInvertedIndex = Env.getCurrentInvertedIndex();
+        Map<Long, Long> tableIdToTotalDeltaNumRows = Maps.newHashMap();
         // try to finish the transaction, if failed just retry in next loop
         for (TransactionState transactionState : readyTransactionStates) {
-            Map<Long, PublishVersionTask> transTasks = transactionState.getPublishVersionTasks();
-            Set<Long> publishErrorReplicaIds = Sets.newHashSet();
-            List<PublishVersionTask> unfinishedTasks = Lists.newArrayList();
-            for (PublishVersionTask publishVersionTask : transTasks.values()) {
-                if (publishVersionTask.isFinished()) {
-                    // sometimes backend finish publish version task,
-                    // but it maybe failed to change transactionid to version for some tablets
-                    // and it will upload the failed tabletinfo to fe and fe will deal with them
-                    List<Long> errorTablets = publishVersionTask.getErrorTablets();
-                    if (errorTablets == null || errorTablets.isEmpty()) {
-                        continue;
-                    } else {
-                        for (long tabletId : errorTablets) {
-                            // tablet inverted index also contains rollingup index
-                            // if tablet meta not contains the tablet, skip this tablet because this tablet is dropped
-                            // from fe
-                            if (tabletInvertedIndex.getTabletMeta(tabletId) == null) {
-                                continue;
-                            }
-                            Replica replica = tabletInvertedIndex.getReplica(
-                                    tabletId, publishVersionTask.getBackendId());
-                            if (replica != null) {
-                                publishErrorReplicaIds.add(replica.getId());
-                            } else {
-                                LOG.info("could not find related replica with tabletid={}, backendid={}",
-                                        tabletId, publishVersionTask.getBackendId());
-                            }
+            Stream<PublishVersionTask> publishVersionTaskStream = transactionState
+                    .getPublishVersionTasks()
+                    .values()
+                    .stream()
+                    .peek(task -> {
+                        if (task.isFinished() && CollectionUtils.isEmpty(task.getErrorTablets())) {
+                            Map<Long, Long> tableIdToDeltaNumRows =
+                                    task.getTableIdToDeltaNumRows();
+                            tableIdToDeltaNumRows.forEach((tableId, numRows) -> {
+                                tableIdToTotalDeltaNumRows
+                                        .computeIfPresent(tableId, (id, orgNumRows) -> orgNumRows + numRows);
+                                tableIdToTotalDeltaNumRows.putIfAbsent(tableId, numRows);
+                            });
                         }
-                    }
-                } else {
-                    unfinishedTasks.add(publishVersionTask);
-                }
-            }
-
-            boolean shouldFinishTxn = false;
-            if (!unfinishedTasks.isEmpty()) {
-                shouldFinishTxn = isAllBackendsOfUnfinishedTasksDead(unfinishedTasks);
-                if (transactionState.isPublishTimeout() || shouldFinishTxn) {
-                    // transaction's publish is timeout, but there still has unfinished tasks.
-                    // we need to collect all error replicas, and try to finish this txn.
-                    for (PublishVersionTask unfinishedTask : unfinishedTasks) {
-                        // set all replicas in the backend to error state
-                        List<TPartitionVersionInfo> versionInfos = unfinishedTask.getPartitionVersionInfos();
-                        Set<Long> errorPartitionIds = Sets.newHashSet();
-                        for (TPartitionVersionInfo versionInfo : versionInfos) {
-                            errorPartitionIds.add(versionInfo.getPartitionId());
-                        }
-                        if (errorPartitionIds.isEmpty()) {
-                            continue;
-                        }
-
-                        Database db = Env.getCurrentInternalCatalog()
-                                .getDbNullable(transactionState.getDbId());
-                        if (db == null) {
-                            LOG.warn("Database [{}] has been dropped.", transactionState.getDbId());
-                            continue;
-                        }
-
-                        for (long tableId : transactionState.getTableIdList()) {
-                            Table table = db.getTableNullable(tableId);
-                            if (table == null || table.getType() != Table.TableType.OLAP) {
-                                LOG.warn("Table [{}] in database [{}] has been dropped.", tableId, db.getFullName());
-                                continue;
-                            }
-                            OlapTable olapTable = (OlapTable) table;
-                            olapTable.readLock();
-                            try {
-                                for (Long errorPartitionId : errorPartitionIds) {
-                                    Partition partition = olapTable.getPartition(errorPartitionId);
-                                    if (partition != null) {
-                                        List<MaterializedIndex> materializedIndexList
-                                                = partition.getMaterializedIndices(MaterializedIndex.IndexExtState.ALL);
-                                        for (MaterializedIndex materializedIndex : materializedIndexList) {
-                                            for (Tablet tablet : materializedIndex.getTablets()) {
-                                                Replica replica = tablet.getReplicaByBackendId(
-                                                        unfinishedTask.getBackendId());
-                                                if (replica != null) {
-                                                    publishErrorReplicaIds.add(replica.getId());
-                                                }
-                                            }
-                                        }
-                                    }
-                                }
-                            } finally {
-                                olapTable.readUnlock();
-                            }
-                        }
-                    }
-                    shouldFinishTxn = true;
-                }
-            } else {
-                // all publish tasks are finished, try to finish this txn.
-                shouldFinishTxn = true;
-            }
+                    });
+            boolean hasBackendAliveAndUnfinishedTask = publishVersionTaskStream
+                    .anyMatch(task -> !task.isFinished() && infoService.checkBackendAlive(task.getBackendId()));
+            transactionState.setTableIdToTotalNumDeltaRows(tableIdToTotalDeltaNumRows);
 
+            boolean shouldFinishTxn = !hasBackendAliveAndUnfinishedTask || transactionState.isPublishTimeout();
             if (shouldFinishTxn) {
                 try {
                     // one transaction exception should not affect other transaction
                     globalTransactionMgr.finishTransaction(transactionState.getDbId(),
-                            transactionState.getTransactionId(), publishErrorReplicaIds);
+                            transactionState.getTransactionId());
                 } catch (Exception e) {
                     LOG.warn("error happens when finish transaction {}", transactionState.getTransactionId(), e);
                 }
@@ -248,8 +160,7 @@ public class PublishVersionDaemon extends MasterDaemon {
                     // if finish transaction state failed, then update publish version time, should check
                     // to finish after some interval
                     transactionState.updateSendTaskTime();
-                    LOG.debug("publish version for transaction {} failed, has {} error replicas during publish",
-                            transactionState, publishErrorReplicaIds.size());
+                    LOG.debug("publish version for transaction {} failed", transactionState);
                 }
             }
 
@@ -258,7 +169,7 @@ public class PublishVersionDaemon extends MasterDaemon {
                     AgentTaskQueue.removeTask(task.getBackendId(), TTaskType.PUBLISH_VERSION, task.getSignature());
                 }
                 if (MetricRepo.isInit) {
-                    long publishTime = transactionState.getPublishVersionTime() - transactionState.getCommitTime();
+                    long publishTime = transactionState.getLastPublishVersionTime() - transactionState.getCommitTime();
                     MetricRepo.HISTO_TXN_PUBLISH_LATENCY.update(publishTime);
                 }
             }
