@@ -109,16 +109,12 @@ public class CatalogTestUtil {
         diskInfo.setAvailableCapacityB(2 << 40); // 1TB
         diskInfo.setTotalCapacityB(2 << 40);
         diskInfo.setDataUsedCapacityB(2 << 10);
-        backend1.setOwnerClusterName(SystemInfoService.DEFAULT_CLUSTER);
         backend1.setDisks(ImmutableMap.of("disk1", diskInfo));
-        backend2.setOwnerClusterName(SystemInfoService.DEFAULT_CLUSTER);
         backend2.setDisks(ImmutableMap.of("disk1", diskInfo));
-        backend3.setOwnerClusterName(SystemInfoService.DEFAULT_CLUSTER);
         backend3.setDisks(ImmutableMap.of("disk1", diskInfo));
         Env.getCurrentSystemInfo().addBackend(backend1);
         Env.getCurrentSystemInfo().addBackend(backend2);
         Env.getCurrentSystemInfo().addBackend(backend3);
-        env.initDefaultCluster();
 
         Database db = createSimpleDb(testDbId1, testTableId1, testPartitionId1, testIndexId1, testTabletId1,
                 testStartVersion);
@@ -260,7 +256,8 @@ public class CatalogTestUtil {
 
         // index
         MaterializedIndex index = new MaterializedIndex(testIndexId2, IndexState.NORMAL);
-        TabletMeta tabletMeta = new TabletMeta(testDbId1, testTableId2, testPartitionId2, testIndexId2, 0, TStorageMedium.HDD);
+        TabletMeta tabletMeta = new TabletMeta(testDbId1, testTableId2, testPartitionId2, testIndexId2, 0,
+                TStorageMedium.HDD);
         index.addTablet(tablet, tabletMeta);
 
         tablet.addReplica(replica);
@@ -333,4 +330,48 @@ public class CatalogTestUtil {
         backend.setAlive(true);
         return backend;
     }
+
+    public static long getTabletDataSize(long tabletId) {
+        Env env = Env.getCurrentEnv();
+        TabletInvertedIndex invertedIndex = env.getTabletInvertedIndex();
+        TabletMeta tabletMeta = invertedIndex.getTabletMeta(tabletId);
+        if (tabletMeta == null) {
+            return -1L;
+        }
+
+        long dbId = tabletMeta.getDbId();
+        long tableId = tabletMeta.getTableId();
+        long partitionId = tabletMeta.getPartitionId();
+        long indexId = tabletMeta.getIndexId();
+        Database db = env.getInternalCatalog().getDbNullable(dbId);
+        if (db == null) {
+            return -1L;
+        }
+        Table table = db.getTableNullable(tableId);
+        if (table == null) {
+            return -1L;
+        }
+        if (table.getType() != Table.TableType.OLAP) {
+            return -1L;
+        }
+        OlapTable olapTable = (OlapTable) table;
+        olapTable.readLock();
+        try {
+            Partition partition = olapTable.getPartition(partitionId);
+            if (partition == null) {
+                return -1L;
+            }
+            MaterializedIndex materializedIndex = partition.getIndex(indexId);
+            if (materializedIndex == null) {
+                return -1L;
+            }
+            Tablet tablet = materializedIndex.getTablet(tabletId);
+            if (tablet == null) {
+                return -1L;
+            }
+            return tablet.getDataSize(true);
+        } finally {
+            olapTable.readUnlock();
+        }
+    }
 }
