@@ -26,6 +26,7 @@ import org.apache.doris.load.DppConfig;
 import org.apache.doris.mysql.privilege.UserProperty;
 
 import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 import mockit.Expectations;
 import mockit.Mocked;
 import org.junit.Assert;
@@ -116,6 +117,7 @@ public class UserPropertyTest {
         Assert.assertEquals(new String[]{"rule1", "rule2"}, userProperty.getSqlBlockRules());
         Assert.assertEquals(2, userProperty.getCpuResourceLimit());
         Assert.assertEquals(500, userProperty.getQueryTimeout());
+        Assert.assertEquals(Sets.newHashSet(), userProperty.getCopiedResourceTags());
 
         // fetch property
         List<List<String>> rows = userProperty.fetchProperty();
